module CustomMoney
  module ClassMethods
    def acts_as_money(*symbols)
      symbols.each do |symbol|
        define_method symbol do
          raw_data_to_money(read_attribute(symbol))
        end

        define_method "#{symbol}=" do |new_value|
          write_attribute(symbol, round_off_raw_data(new_value))
        end
      end
    end

    def validate_currency(*symbols)
      symbols.each do |symbol|
        define_method symbol do
          validate_curr(read_attribute(symbol))
        end

        define_method "#{symbol}=" do |new_value|
          write_attribute(symbol, new_value)
        end
      end
    end

  end

  module InstanceMethods
    def raw_data_to_money(value)
      Money.new(value) rescue nil
    end

    def round_off_raw_data(value)
      raw_data_to_money(value).try(:to_d)
    end

    def validate_curr(value)
      errors.add(:currency, "Invalid currency - #{value}") if (Money::Currency.find(value)).nil?
      value
    end
  end

  def self.included(receiver)
    receiver.extend         ClassMethods
    receiver.send :include, InstanceMethods
  end
end
