require_relative '../../lib/sc_core/supply_chain'

module ScCore
  module CommandHandler
    def handles(command, opts={})
      MessageBus.subscribe command, self, opts[:with] || command
    end
  end
  
  module EventHandler
    def subscribes(event, opts={})
      MessageBus.subscribe event, self, opts[:with] || event
    end
  end
  
  class MessageBusError < StandardError
    def initialize(msg)
      super
    end
  end
  
  class MessageBus
    @handlers = {}
  
    class << self
      
      attr_reader :handlers

      def subscribe(message, handler, method)
        @handlers[message] ||= []
        @handlers[message] << { :handler => handler, :method => method }
      end
  
      #
      # opts -- :async => true/false - Decides whether the message should be asynchronously handled.
      #
      def send(message, args, opts={})
        async?(opts) ? async_send(message, args, opts) : sync_send(message, args, opts)
      end
      
      def publish(message, args, opts={})
        async?(opts) ? async_publish(message, args, opts) : sync_publish(message, args, opts)
      end
      
      def has_handlers?(message)
        @handlers[message] ||= []
        @handlers[message].size > 0
      end
      
      private
      def async?(opts)
        opts = opts || {}
        opts[:async]
      end 
      
      def sync_send(message, args, opts)
        return if !@handlers.include? message || @handlers[message].size == 0
        handler = @handlers[message].first
        handler[:handler].send handler[:method], args
      end
      
      def sync_publish(message, args, opts)
        @handlers[message] ||= []
        @handlers[message].each do |handler|
          handler[:handler].send handler[:method], args
        end
      end
      
      def async_send(message, args, opts)
        raise MessageBusError.new('Missing input') unless args[:request_uri] && args[:http_method] && args[:payload] && args[:exchange_name]
        mq_client = ScMq::Client::MqClient.new(args[:sender_app_id] || SupplyChain.app_id, args[:exchange_name])
        mq_client.send_message(args[:payload], args[:http_method], args[:request_uri], args[:context] || {})
      end
      
      def async_publish(message, args, opts)
        raise MessageBusError.new('Missing input') unless args[:tag] && args[:payload]
        args[:payload][:event] = message
        group_id = args[:group_id]
        mq_client = ScMq::Client::MqClient.new(args[:sender_app_id] || SupplyChain.app_id, args[:tag], 'topic')
        mq_client.send_message(args[:payload].to_json, nil, nil, {}, nil, nil, args[:tag], nil, group_id)
      end
      
    end
  end
end