require 'rubygems'
require 'test/unit'
require 'mocha'

require_relative '../lib/sc_metrics'

class MethodProfilerTest < Test::Unit::TestCase
  include Mocha::API

  def test_method_profiler_should_delegate_the_method_to_give_object
    object = mock {expects(:foo).with(1).returns(1)}
    ScMetrics.expects(:timing).with do |key, time|
      key == "method_profiler.#{object.class}.foo"
    end
    assert_equal 1, ScMetrics::MethodProfiler.new(object).foo(1)
  end

  def test_method_profiler_when_the_method_raises_an_exception
    object = mock {expects(:foo).with(1).raises(RuntimeError)}
    ScMetrics.expects(:timing).never
      
    assert_raises(RuntimeError) do
      ScMetrics::MethodProfiler.new(object).foo(1)
    end
  end

  def test_method_profiler_when_method_does_not_exist
    ScMetrics.expects(:timing).never
      
    assert_raises(NoMethodError) do
      ScMetrics::MethodProfiler.new('dfdfd').foo
    end
  end

  def test_method_profiler_with_a_block
    ScMetrics.stubs(:timing)

    assert_equal [3, 2, 1], ScMetrics::MethodProfiler.new([1, 2, 3]).sort_by {|x| -x}
  end
end
