require_relative 'sc_client.rb'

module SC
  module Client
    class OMSClient
      @@ORDER_ID = "ORDER_ID"
      @@ORDER_ITEM_ID = "ORDER_ITEM_ID"
      @@PAYMENT_ID = "PAYMENT_ID"
      @@ACTION_ID = "ACTION_ID"
      @@PARAM_STRING = "PARAM_STRING"
      @@NOTE_ID = "NOTE_ID"
      @@CREATE_ORDER_RESOURCE = "/orders"
      @@GET_ORDER_RESOURCE =  "/orders/" + @@ORDER_ID
      @@DO_ACTION_ON_ORDER_RESOURCE =  "/orders/" + @@ORDER_ID + "/" + @@ACTION_ID
      @@DO_ACTION_ON_ORDER_ITEM_RESOURCE =  "/order_items/" + @@ORDER_ITEM_ID + "/" + @@ACTION_ID
      @@FIND_ORDERS_RESOURCE =  "/orders?" + @@PARAM_STRING
      @@CREATE_ORDER_ADJUSMENT_RESOURCE =  "/orders/" + @@ORDER_ID + "/adjustments/"
      @@CREATE_ORDER_ITEM_ADJUSMENT_RESOURCE =  "/order_items/" + @@ORDER_ITEM_ID + "/adjustments/"
      @@CREATE_ORDER_ITEM_RESOURCE = "/orders/" + @@ORDER_ID + "/order_items/"
      @@GET_ORDER_ITEMS_RESOURCE =  "/orders/" + @@ORDER_ID + "/order_items/"
      @@FIND_ORDER_ITEMS_RESOURCE =  "/order_items?" + @@PARAM_STRING
      @@GET_ORDER_ITEM_RESOURCE =  "/order_items/" + @@ORDER_ITEM_ID
      @@ADD_PAYMENT_RESOURCE =  "/orders/" + @@ORDER_ID + "/payments/"
      @@GET_PAYMENTS_RESOURCE =  "/orders/" + @@ORDER_ID + "/payments/"
      @@DO_ACTION_ON_PAYMENT_RESOURCE = "/orders/" + @@ORDER_ID + "/payments/" + @@PAYMENT_ID + "/" + @@ACTION_ID
      @@CREATE_ORDER_NOTE_RESOURCE =  "/orders/" + @@ORDER_ID + "/notes/"
      @@GET_ORDER_NOTES_RESOURCE = "/orders/" + @@ORDER_ID + "/notes?" + @@PARAM_STRING
      @@GET_ORDER_NOTE_BY_NOTE_ID_RESOURCE = "/orders/" + @@ORDER_ID + "/notes/" + @@NOTE_ID
      @@CREATE_ORDER_ITEM_NOTE_RESOURCE =  "/order_items/" + @@ORDER_ITEM_ID + "/notes/"
      @@GET_ORDER_ITEM_NOTES_RESOURCE = "/order_items/" + @@ORDER_ITEM_ID + "/notes?" + @@PARAM_STRING
      @@GET_ORDER_ITEM_NOTE_BY_NOTE_ID_RESOURCE = "/order_items/" + @@ORDER_ITEM_ID + "/notes/" + @@NOTE_ID
      @@AUTO_APPROVE_DUPLICATE_ORDERS_RESOURCE = "/order_items/auto_approve_duplicate_orders"
      @@AUTO_HOLD_COD_ORDERS_RESOURCE = "/orders/auto_hold_cod_orders"


      def initialize(client)
        @client = client
      end

      def create_order(payload, headers={:content_type => :json})
        @client.post(@@CREATE_ORDER_RESOURCE, payload, headers)

      end

      def get_order_by_id(order_id, headers={:content_type => :json})
        resourceLocation = @@GET_ORDER_RESOURCE.sub(@@ORDER_ID, order_id.to_s)
        @client.get(resourceLocation, headers)
      end

      def find_orders(params="", headers={:content_type => :json})
        resource_location = @@FIND_ORDERS_RESOURCE.sub(@@PARAM_STRING, params)
        puts "Finding Orders using #{resource_location}"
        response = @client.get(resource_location, headers)
        puts "Find Order response #{response.inspect}"
        response
      end

      def create_order_adjustment(order_id, payload, headers={:content_type => :json})
        resourceLocation = @@CREATE_ORDER_ADJUSMENT_RESOURCE.sub(@@ORDER_ID, order_id.to_s)
        @client.post(resourceLocation, payload, headers)
      end

      def do_action_on_order_by_id(order_id, action, payload = {}, headers={:content_type => :json})
        resource_location =  @@DO_ACTION_ON_ORDER_RESOURCE.sub(@@ORDER_ID, order_id.to_s).sub(@@ACTION_ID, action.to_s)
        @client.put(resource_location, payload, headers)
      end

      def create_order_item_adjustment(order_item_id, payload, headers={:content_type => :json})
        resourceLocation = @@CREATE_ORDER_ITEM_ADJUSMENT_RESOURCE.sub(@@ORDER_ITEM_ID, order_item_id.to_s)
        @client.post(resourceLocation, payload, headers)
      end

      def create_order_item(order_id, payload, headers={:content_type => :json})
        resourceLocation = @@CREATE_ORDER_ITEM_RESOURCE.sub(@@ORDER_ID, order_id.to_s)
        @client.post(resourceLocation, payload, headers)
      end

      def get_order_items(order_id, payload, headers={:content_type => :json})
        resourceLocation = @@GET_ORDER_ITEMS_RESOURCE.sub(@@ORDER_ID, order_id.to_s)
        @client.get(resourceLocation, headers)
      end

      def get_order_item_by_id(order_item_id, headers={:content_type => :json})
        resourceLocation = @@GET_ORDER_ITEM_RESOURCE.sub(@@ORDER_ITEM_ID, order_item_id.to_s)
        @client.get(resourceLocation, headers)
      end

      def find_order_items(params="", headers={:content_type => :json})
        resourceLocation = @@FIND_ORDER_ITEMS_RESOURCE.sub(@@PARAM_STRING, params)
        puts "Finding order items using resource #{resourceLocation}"
        response  = @client.get(resourceLocation, headers)
        puts "Find Order Items response #{response.inspect}"
        return response
      end

      def do_action_on_order_item_by_id(order_item_id, action, payload = {}, headers={:content_type => :json})
        resource_location =  @@DO_ACTION_ON_ORDER_ITEM_RESOURCE.sub(@@ORDER_ITEM_ID, order_item_id.to_s).sub(@@ACTION_ID, action.to_s)
        @client.put(resource_location, payload, headers)
      end

      def add_payment_for_order_id(order_id, payload, headers={:content_type => :json})
        resourceLocation = @@ADD_PAYMENT_RESOURCE.sub(@@ORDER_ID, order_id.to_s)
        @client.post(resourceLocation, payload, headers)
      end

      def get_payments_for_order_id(order_id, headers={:content_type => :json})
        resourceLocation = @@GET_PAYMENTS_RESOURCE.sub(@@ORDER_ID, order_id.to_s)
        @client.get(resourceLocation, headers)
      end

      def do_action_on_payment_id_for_order_id(order_id, payment_id, action, payload, headers={:content_type => :json})
        resource_location =  @@DO_ACTION_ON_PAYMENT_RESOURCE.sub(@@ORDER_ID, order_id.to_s).sub(@@PAYMENT_ID, payment_id.to_s).sub(@@ACTION_ID, action.to_s)
        @client.put(resource_location, payload, headers)
      end

      def create_order_note(order_id, payload, headers={:content_type => :json})
        resourceLocation = @@CREATE_ORDER_NOTE_RESOURCE.sub(@@ORDER_ID, order_id.to_s)
        @client.post(resourceLocation, payload, headers)
      end

      def get_order_notes(order_id, params="", headers={:content_type => :json})
        resourceLocation = @@GET_ORDER_NOTES_RESOURCE.sub(@@ORDER_ID, order_id.to_s).sub(@@PARAM_STRING, params)
        @client.get(resourceLocation, headers)
      end

      def get_order_note_by_note_id(order_id, note_id, headers={:content_type => :json})
        resourceLocation = @@GET_ORDER_NOTE_BY_NOTE_ID_RESOURCE.sub(@@ORDER_ID, order_id.to_s).sub(@@NOTE_ID, note_id.to_s)
        @client.get(resourceLocation, headers)
      end

      def create_order_item_note(order_item_id, payload, headers={:content_type => :json})
        resourceLocation = @@CREATE_ORDER_ITEM_NOTE_RESOURCE.sub(@@ORDER_ITEM_ID, order_item_id.to_s)
        @client.post(resourceLocation, payload, headers)
      end

      def get_order_item_notes(order_item_id, params="", headers={:content_type => :json})
        resourceLocation = @@GET_ORDER_ITEM_NOTES_RESOURCE.sub(@@ORDER_ITEM_ID, order_item_id.to_s).sub(@@PARAM_STRING, params)
        @client.get(resourceLocation, headers)
      end

      def get_order_item_note_by_note_id(order_item_id, note_id, headers={:content_type => :json})
        resourceLocation = @@GET_ORDER_ITEM_NOTE_BY_NOTE_ID_RESOURCE.sub(@@ORDER_ITEM_ID, order_item_id.to_s).sub(@@NOTE_ID, note_id.to_s)
        @client.get(resourceLocation, headers)
      end

      def cancel_order(order_id, payload, headers={:content_type => :json})
        resource_location = "/orders/#{order_id.to_s}/request_cancellation"
        puts "Cancelling Order using resource #{resource_location}"
        response = @client.put(resource_location, payload, headers)
        puts "Cancel Order response #{response}"
      end

      def cancel_order_item(order_item_id, payload, headers={:content_type => :json})
        resource_location = "/order_items/#{order_item_id.to_s}/request_cancellation"
        puts "Cancelling Order item using resource #{resource_location}"
        response = @client.put(resource_location, payload, headers)
        puts "Cancel Order item response #{response}"
      end

      def do_action_on_order_by_id(order_id, action, payload = {}, headers={:content_type => :json})
        resource_location =  @@DO_ACTION_ON_ORDER_RESOURCE.sub(@@ORDER_ID, order_id.to_s).sub(@@ACTION_ID, action.to_s)
        @client.put(resource_location, payload, headers)
      end

      def auto_approve_duplicate_orders(payload = {}, headers={:content_type => :json})
        @client.post(@@AUTO_APPROVE_DUPLICATE_ORDERS_RESOURCE, payload, headers)
      end

      def auto_hold_cod_orders(payload = {}, headers={:content_type => :json})
        @client.post(@@AUTO_HOLD_COD_ORDERS_RESOURCE, payload, headers)
      end

    end



  end
end