#require_relative 'sc_client.rb'

module SC
  module Client
    class ProcurementClient

      @@requirement = "/requirements"
      @@purchase_orders = "/purchase_orders"
      @@purchase_order_items = "/purchase_order_items"
      @@cancellation = "/procurement/cancel"
      @@hold = "/requirements/hold"

      attr_reader :client, :base_url

      def initialize(client)
        @client = client
      end

      def create_requirement(request, headers={:content_type => :json})
        @client.post @@requirement, request, headers
      end

      def get_requirements()
        resource = "/requirements"
        puts "Getting requirements using #{resource}"
        response = @client.get resource,{}
        puts "Get Requirements response #{response.inspect}"
        response
      end

      def search_requirements(query_params,headers={:content_type => :json})
        @client.get "/requirements?#{query_params}"
      end

      def approve_requirements(ids, headers={:content_type => :json})
        resource = @@requirement + "/" + ids + "/approve"
        puts "Approving requirements using resource #{resource}"
        response = @client.post resource, headers
        puts "Approve Requirement response #{response}"
        response
      end

      def create_purchase_order(request, headers={:content_type => :json})
        puts "Creating Purchase Orders using resource #{@@purchase_orders} and request body #{request}"
        response = @client.post @@purchase_orders, request, headers
        puts "Create Purchase Order Response #{response}"
        response
      end

      def receive_against_purchase_order(id, request, headers={:contenty_type => :json})
        @client.post @@purchase_orders + "/" + id.to_s + "/receive", request, headers
      end

      def get_purchase_orders(ids, headers={:content_type => :json})
        resource =  @@purchase_orders+"/"+ ids * ","
        puts "Getting purchase orders using resource #{resource}"
        response = @client.get resource, headers
        puts "Get purchase orders response #{response.inspect}"
        response
      end

      def get_purchase_order_item(po_id, po_item_id, headers={:content_type => :json})
        resource =  @@purchase_orders+"/"+po_id.to_s+"/purchase_order_items/" + po_item_id.to_s
        puts "Getting purchase order items using resource #{resource}"
        response = @client.get resource, headers
        puts "Get purchase order items response #{response.inspect}"
        response
      end

      def search_purchase_order_items(params={})
        url_param = encode_params(params)
        @client.get "/purchase_order_items?" + url_param.chop
      end

      def search_purchase_order_item(po_item_id)
        @client.get "/purchase_order_items" + "/" + po_item_id.to_s
      end

      def find_po(params={})
        @client.get "/purchase_orders#{"?"+encode_params(params).chop unless params.nil?}"
      end

      def search_po_and_po_items_for(ids, headers={:content_type => :json})
        @client.get @@purchase_orders+"/"+ids.to_s, headers
      end

      def cancel(request, headers={:content_type => :json})
        @client.post @@cancellation, request, headers
      end

      def cancel_po_items(request, headers={:content_type => :json})
        @client.post @@purchase_order_items+"/cancel", request, headers
      end

      def hold_requirements(request,headers={:content_type => :json})
        @client.post @@hold, request, headers
      end

      def encode_params(params)
        url_param = ""
        params.each { |k, v| url_param += "#{k}=#{v}&" }
        puts url_param
        return url_param
      end
    end
  end
end

