module SC
  module Config
    class SetupConfig

      SERVICE_PORTS ={
          :b2c => {
              :oms => 3001,
              :customer => 3101,
              :payment => 3201,
              :fulfillment => 3301,
              :proc => 3401,
              :warehouse => 3501,
              :warehouseui => 3100,
              :shipping => 3601,
              :doc => 3701,
              :geocourier => 3801,
              :nss => 3901
          },
          :b2b => {
              :oms => 4001,
              :customer => 4101,
              :payment => 4201,
              :fulfillment => 4301,
              :proc => 4401,
              :warehouse => 4501,
              :warehouseui => 4100,
              :shipping => 4601,
              :doc => 4701,
              :geocourier => 4801,
              :nss => 4901
          }
      }

      def initialize(env, prefix="")
        @prefix = prefix
        @env = env.to_s
      end

      def oms_b2c_db_connection_string
        db_connection_string :oms,:b2c
      end

      def customer_b2c_db_connection_string
        db_connection_string :customer,:b2c
      end

      def payment_b2c_db_connection_string
        db_connection_string :payment,:b2c
      end

      def warehouse_b2c_db_connection_string
        db_connection_string :warehouse,:b2c
      end

      def shipping_b2c_db_connection_string
        db_connection_string :shipping,:b2c
      end

      def oms_b2c_service_base_url
        service_base_url :oms, :b2c
      end

      def customer_b2c_service_base_url
        service_base_url :customer, :b2c
      end

      def payment_b2c_service_base_url
        service_base_url :payment, :b2c
      end

      def fulfillment_b2c_service_base_url
        service_base_url :fulfillment, :b2c
      end

      def proc_b2c_service_base_url
        service_base_url :proc, :b2c
      end

      def warehouse_b2c_service_base_url
        service_base_url :warehouse, :b2c
      end

      def shipping_b2c_service_base_url
        service_base_url :shipping, :b2c
      end

      def doc_b2c_service_base_url
        service_base_url :doc, :b2c
      end

      def service_base_url(service, supply_chain_company)
        "http://#{service_host service, supply_chain_company}:#{SERVICE_PORTS[supply_chain_company][service]}"
      end

      def queue_name(service, supply_chain_company)
        "#{service.to_s}_#{supply_chain_company.to_s}_#{@env}.fanout.queue"
      end

      def exchange_name(service, supply_chain_company)
        "#{service.to_s}_#{supply_chain_company.to_s}_#{@env}.fanout.exchange"
      end

      def db_connection_string(service, supply_chain_company)
        {
            :host => service_db_host(service.to_s, supply_chain_company.to_s),
            :port => 3306,
            :username=>'root',
            :password=>'',
            :db=> "#{service.to_s}_#{supply_chain_company.to_s}_#{@env}"
        }
      end

      #if required prepare actual host names using prefix, env and service name
      def service_host(service, supply_chain_company=:b2c)
        "#{@env}.#{supply_chain_company.to_s}.#{service.to_s}.flipkart.com"
      end

      #if required prepare actual host names using prefix, env and service name
      def service_db_host(service, supply_chain_company=:b2c)
        "#{@env}.#{supply_chain_company.to_s}.#{service.to_s}.db.flipkart.com"
      end

    end
  end
end
