require 'hashie'
module SC
  module Warehouse
    module Layout

      def warehouses
        Hashie::Mash.new({
                             :del => {
                                 :id => "del",
                                 :name => "Delhi Warheouse",
                                 :address => {
                                     :address1 => "Khasra No. 435,Road No.# 04",
                                     :address2 => "Lal Dora Ext. Mahipalpur",
                                     :city => "New Delhi",
                                     :state => "New Delhi",
                                     :postal_code => 110037,
                                     :country => "India"
                                 }
                             },
                             :blr => {
                                 :id => "blr",
                                 :name => "Bangalore Warehouse",
                                 :address => {
                                     :address1 => "Crescent #148, 2nd Floor,",
                                     :address2 => "Floor, 	Next to Gopalan Mall, Mysore Road,",
                                     :city => "Bangalore",
                                     :state => "Karnataka",
                                     :postal_code => 560026,
                                     :country => "India"
                                 }
                             }
                         })
      end

      def storage_zones
        Hashie::Mash.new({
                             :forward_area_book => {
                                 :name => "forward_area_book",
                                 :area => "forward_area",
                                 :is_fast_moving => nil,
                                 :is_book => true
                             },
                             :forward_area_non_book => {
                                 :name => "forward_area_non_book",
                                 :area => "forward_area",
                                 :is_fast_moving => nil,
                                 :is_book => false
                             },
                             :store_book_fast_moving => {
                                 :name => "store_book_fast_moving",
                                 :area => "store",
                                 :is_fast_moving => true,
                                 :is_book => true
                             },
                             :store_book_slow_moving => {
                                 :name => "store_book_slow_moving",
                                 :area => "store",
                                 :is_fast_moving => false,
                                 :is_book => true
                             },
                             :store_non_book_fast_moving => {
                                 :name => "store_non_book_fast_moving",
                                 :area => "store",
                                 :is_fast_moving => true,
                                 :is_book => false
                             },
                             :store_non_book_slow_moving => {
                                 :name => "store_non_book_slow_moving",
                                 :area => "store",
                                 :is_fast_moving => false,
                                 :is_book => false
                             },
                             :default => {
                                 :name => "default_zone",
                                 :area => "store",
                                 :is_fast_moving => nil,
                                 :is_book => nil
                             }
                         })

      end

      def picking_zones
        Hashie::Mash.new({
                             :forward_area_picking_zone_1 => {
                                 :name => "forward_area_picking_zone_1",
                                 :area => "forward_area"
                             },
                             :forward_area_picking_zone_2 => {
                                 :name => "forward_area_picking_zone_2",
                                 :area => "forward_area"
                             },
                             :forward_area_picking_zone_3 => {
                                 :name => "forward_area_picking_zone_3",
                                 :area => "forward_area"
                             },
                             :store_picking_zone_1 => {
                                 :name => "store_picking_zone_1",
                                 :area => "store"
                             },
                             :store_picking_zone_2 => {
                                 :name => "store_picking_zone_2",
                                 :area => "store"
                             },
                             :store_picking_zone_3 => {
                                 :name => "store_picking_zone_3",
                                 :area => "store"
                             },
                             :store_picking_zone_4 => {
                                 :name => "store_picking_zone_4",
                                 :area => "store"
                             },
                             :store_picking_zone_5 => {
                                 :name => "store_picking_zone_5",
                                 :area => "store"
                             },
                             :store_picking_zone_6 => {
                                 :name => "store_picking_zone_6",
                                 :area => "store"
                             }


                         })
      end

      def storage_locations
        fa_locations = populate_locations((1..4), "forward_area", 10, "forward_area", "fa", :forward_area_book, :forward_area_picking_zone_1).
                merge(populate_locations((5..5), "forward_area", 10, "forward_area", "fa", :forward_area_book, :forward_area_picking_zone_2)).
                merge(populate_locations((6..6), "forward_area", 10, "forward_area", "fa", :forward_area_non_book, :forward_area_picking_zone_2)).
                merge(populate_locations((7..10), "forward_area", 10, "forward_area", "fa", :forward_area_non_book, :forward_area_picking_zone_3))

        store_locations = populate_locations((1..4), "store", 10, "store", "st", :store_book_fast_moving, :store_picking_zone_1).
                merge(populate_locations((5..5), "store", 10, "store", "st", :store_book_fast_moving, :store_picking_zone_2)).
                merge(populate_locations((6..6), "store", 10, "store", "st", :store_book_slow_moving, :store_picking_zone_2)).
                merge(populate_locations((7..10), "store", 10, "store", "st", :store_book_slow_moving, :store_picking_zone_3)).

                merge(populate_locations((11..14), "store", 10, "store", "st", :store_non_book_fast_moving, :store_picking_zone_4)).
                merge(populate_locations((15..15), "store", 10, "store", "st", :store_non_book_fast_moving, :store_picking_zone_5)).
                merge(populate_locations((16..16), "store", 10, "store", "st", :store_non_book_slow_moving, :store_picking_zone_5)).
                merge(populate_locations((17..20), "store", 10, "store", "st", :store_non_book_slow_moving, :store_picking_zone_6)).
                merge(populate_locations((21..21), "store", 10, "store", "st", nil, nil))

        Hashie::Mash.new({
                             :forward_area => fa_locations,
                             :store => store_locations,
                             :inward_fa => populate_locations((1..1), "inward_fa", 1000, "forward_area"),
                             :inward_bulk => populate_locations((1..1), "inward_bulk", 1000, "store"),
                             :packing_table => populate_locations((1..1), "packing_table", 1000)

                         })
      end

      private
      def populate_locations(range, location_type, capacity = 10, area = nil, label_prefix = nil, storage_zone = nil, picking_zone = nil)
        locations = Hash.new
        range.each do |index|
          label = label_prefix.nil? ? location_type : "#{label_prefix}_#{index}"
          locations[label] =  {
              :label => label,
              :capacity => capacity,
              :location_type => location_type,
              :area => area,
              :storage_zone => storage_zone,
              :picking_zone => picking_zone
          }
        end
        locations
      end

    end
  end
end