module SC
  module Factory
    module ShippingFactory

      def self.prepare_create_shipment_body(shipment_info, shipment_items_info)
        {
            :origin_party_id        => "del",
            :origin_address_id      => 4563,
            :origin_type            => "WAREHOUSE",
            :destination_address_id => 3209,
            :destination_type       => "CUSTOMER",
            :shipment_items         => shipment_items_info
        }.merge(shipment_info)
      end

      def self.default_multiple_shipment_body
        {
            :origin_party_id        => "del",
            :origin_address_id      => 4563,
            :origin_type            => "WAREHOUSE",
            :destination_address_id => 3209,
            :destination_type       => "CUSTOMER",
            :shipment_items         => [
                prepare_single_shipment_item_body(12345,"FSN1","SKU1",1),
                prepare_single_shipment_item_body(123456,"FSN2","SKU2",2)
            ]
        }
      end

      def self.default_shipment_body
        {
            :origin_party_id        => "del",
            :origin_address_id      => 4563,
            :origin_type            => "WAREHOUSE",
            :destination_address_id => 3209,
            :destination_type       => "CUSTOMER",
            :shipment_items         => [prepare_single_shipment_item_body(12345,"FSN1","SKU1",1)]
        }
      end

      def self.prepare_single_shipment_item_body(order_item_id,fsn,sku,quantity)
        {
            :order_item_id  => order_item_id,
            :fsn              => fsn,
            :sku              => sku,
            :quantity         => quantity,
        }
      end

      def self.default_confirm_shipment_body
        create_confirm_shipment_body 123.456,"1234534"
      end

      def self.create_confirm_shipment_body(weight,tracking_id)
        confirm_shipment_body = {
            :weight       =>  weight,
            :tracking_id  =>  tracking_id
        }
        confirm_shipment_body.delete(:weight) if weight.nil?
        confirm_shipment_body.delete(:tracking_id) if tracking_id.nil?
        return confirm_shipment_body
      end

      # Used for Test Validation
      def self.prepare_expected_create_shipment_json(input_json,shipment_id,shipment_status)
        input_json[:assigned_courier_id] = 1;
        input_json[:id] = shipment_id;
        input_json[:preferred_courier_id] = nil;
        input_json[:shipment_type] = nil;
        input_json[:status] = shipment_status;
        input_json[:tracking_id] = nil;
        input_json[:weight] = nil;
        shipment_items  = input_json[:shipment_items]

        shipment_items.each do |shipment_item|
          shipment_item[:status] = shipment_status
          shipment_item[:shipment_id] = shipment_id
          #  shipment_item[:id] = shipment_id
        end
        return input_json
      end

      def self.prepare_expected_confirm_shipment_json(shipment_json, shipment_id, shipment_status)
        shipment_json["shipment"]["status"] = shipment_status
        shipment_items  = shipment_json["shipment"]["shipment_items"]

        shipment_items.each do |shipment_item|
          shipment_item["status"] = shipment_status
        end
        return shipment_json
      end

    end

  end
end