module SC
  module Factory
    class WarehouseFactory

      #Helper Factory Methods which create Json Body for various Warehouse Operations
      def self.default_pending_reservations_body
        return create_pending_reservations_body("Order_Item_1", "FSN1", "SKU1", 1,1)
      end

      def self.create_pending_reservations_body(order_item_id, fsn, sku, quantity, address_id)
        {
            :order_item_id  => order_item_id,
            :fsn            => fsn,
            :sku            => sku,
            :quantity       => quantity,
            :address_id       => address_id
        }
      end

      def self.default_create_picklist_body(location_type, type, count)
        return create_create_picklist_body("FORWARD_AREA","DEFAULT",1)
      end

      def self.create_create_picklist_body(location_type, picking_zone_id, count=nil)
        body = {
            :location_type  => location_type,
            :picking_zone_id   => picking_zone_id,
        }
        body[:count] = count if not count.nil?
        return body
      end

      def self.create_create_transfer_list_body(picking_zone_id, count=nil)
        body = {
            :picking_zone_id   => picking_zone_id,
        }
        body[:count] = count if not count.nil?
        return body
      end

      def self.default_add_to_inventory_store_body
        create_add_to_inventory_body("FSN1","SKU1",2,"STORE","PO_ID1")
      end

      #in system FORWARD_AREA would transfate to location_type of INWARD_FA and location_label INWARD_FA
      def self.default_add_to_inventory_fa_body
        create_add_to_inventory_body("FSN1","SKU1",2,"FORWARD_AREA","PO_ID1")
      end

      def self.create_add_to_inventory_body(fsn, sku, quantity, location_type, purchase_order_id=nil)
        body = {
            :fsn  => fsn,
            :sku  => sku,
            :quantity  => quantity,
            :location_type  => location_type
        }
        body[:purchase_order_id] = purchase_order_id if not purchase_order_id.nil?
        return body
      end

      def self.default_assign_shelf_body
        create_assign_shelf_body("FSN4","SKU4","INWARD_FA","Shelf4",1)
      end

      def self.create_assign_shelf_body(fsn, sku, source_location_name, destination_location_name, quantity, order_item_id="not_used_item_id")
        request = {
            :fsn  => fsn,
            :sku  => sku,
            :source_location_name  => source_location_name,
            :quantity  => quantity,
            :order_item_id  => order_item_id,
        }
        request[:destination_location_name] = destination_location_name if not destination_location_name.nil?
        return request
      end

      # Used for Test Validation
      def self.prepare_expected_create_pending_inventory_json(input_json,pending_inventory_id,warehouse_id)
        input_json[:id] = pending_inventory_id;
        input_json[:warehouse_id] = warehouse_id;
        return input_json
      end
    end
  end
end