#Mixin of OMSOperations requires
#   : oms_client function implementation in the class which is mixin OMSOperations
#   : customer_client function implementation in the class which is mixin OMSOperations
#   : shelves function to return hash of shelf hashies
module SC
  module Operations
    module OMSOperations
      DEFAULT_ADDRESS = {
                         first_name: "First Name",
                         second_name: "Second Name",
                         line1: "Line 1",
                         line2: "Line 2",
                         city: "Bangalore",
                         state: "Karnataka",
                         pincode: 560026
                        }
      def create_customer!(firstname, lastname, addrline1, addrline2, city, state, pincode)
        customer_body = {
            :external_customer_id=>"C_"+Time.now.to_i.to_s,
            :email_id=>"#{Time.now.to_i.to_s}@gmail.com",
            :default_shipping_address=>{
                :first_name=>firstname,
                :last_name=>lastname,
                :address_line1=>addrline1,
                :address_line2=>addrline2,
                :city=>city,
                :state=>state,
                :country=>"India",
                :pincode=>pincode,
                :phone=>"9999999999",
                :secondary_phone=>"1111111111",
                :active=>"true",
                :guest=>"false"
            },
            :default_billing_address=>{
                :first_name=>firstname,
                :last_name=>lastname,
                :address_line1=>addrline1,
                :address_line2=>addrline2,
                :city=>city,
                :state=>state,
                :country=>"India",
                :pincode=>pincode,
                :phone=>"9999999999",
                :secondary_phone=>"1111111111",
                :active=>"true",
                :guest=>"false"
            }
        }
        response = @customer_client.create_customer(customer_body.to_json)
        puts "Create Customer Response #{response.inspect}"
        raise("Create Customer Failed with reason #{response.body}") if response.code != 201
        customer_body[:id] = JSON(response.body)["customer"]["id"]
        customer_body[:default_billing_address_id] = JSON(response.body)["customer"]["default_billing_address_id"]
        customer_body[:default_shipping_address_id] = JSON(response.body)["customer"]["default_shipping_address_id"]
        customer_body[:billing_address_id] = JSON(response.body)["customer"]["default_billing_address_id"]
        customer_body[:shipping_address_id] = JSON(response.body)["customer"]["default_shipping_address_id"]
        Hashie::Mash.new(customer_body)
      end

      #order_item_info [{:product => product, :quantity => quantity}]
      def create_book_sales_order!(customer, order_items)
        billing_amount = 0.0
        order_body = {
            :external_id=>"OD"+Time.now.to_i.to_s,
            :sales_channel=>"WEB",
            :order_date=>Time.now.to_s.split(" +")[0],
            :created_by=>"website",
            :currency=>"INR",
            :billing_amount=>"700.5",
            :bill_to_party_id=>customer.id,
            :bill_from_party_id=>"WSRETAIL",
            :billing_address_id=>customer.billing_address_id,
            :shipping_address_id=>customer.shipping_address_id,
            :gift_wrap=>"false",
            :guest=>"false",
            :order_items => order_items.collect do |order_item|
              item_price = order_item[:price].nil? ? 700.5 : order_item[:price]
              product   = order_item[:product]
              quantity = order_item[:quantity]
              order_item_price = item_price * quantity
              billing_amount += order_item_price
              {
                  :order_item_type_id=>"physical",
                  :product_id=>product.fsn,
                  :sku => product.sku,
                  :title=>product.product_title,
                  :category_id=>"20002",
                  :quantity=>order_item[:quantity],
                  :list_price=>"1000.0",
                  :selling_price=>"#{item_price}",
                  :sla=>"5",
                  :deliver_before_date=>(Time.now + 60*60*48).to_s,
                  :deliver_on_date=>(Time.now + 60*60*48).to_s,
                  :deliver_after_date=>(Time.now + 60*60*48).to_s,
                  :sale_package=>"",
                  :order_date=>Time.now.to_s.split(" +")[0]
              }
            end
        }
        order_body[:billing_amount] = billing_amount

        response = @oms_client.create_order(order_body.to_json)
        puts "Create Order Response #{response.inspect}"
        raise("Create Order Failed with reason #{response.body}") if response.code != 201
        Hashie::Mash.new(JSON.parse(response.body)["order"])
      end

      def create_and_approve_payment(order, customer)
        add_payment_body = {
            :amount=>order.billing_amount,
            :applied_amount=>order.billing_amount,
            :comments=>"Full Payment",
            :external_id=>order.external_id.sub("OD","EXT"),
            :flipkart_ref_id=>order.external_id,
            :installments=>"1",
            :is_possible_fraud=>"false",
            :party_id_from=>customer.id,
            :party_id_to=>"wsretail",
            :payment_channel=>"HDFC BANK",
            :payment_gateway_error_code=>"0",
            :payment_gateway_info=>"HDFC",
            :payment_gateway_preference_id=>order.external_id.sub("OD","HDFC"),
            :payment_method=>"CREDIT_CARD",
            :payment_ref_num=>"REF-1234",
            :payment_type=>"PAYMENT",
            :status=>"init",
            :user_login=>customer.email_id
        }

        response = @oms_client.add_payment_for_order_id(order.id, add_payment_body.to_json)
        puts "Add Payment Response #{response.inspect}"
        raise("Create Payment Failed with reason #{response.body}") if response.code != 201

        response = @oms_client.do_action_on_payment_id_for_order_id(order.id, JSON(response.body)["payment"]["id"], "receive", nil)
        puts "Receive Payment Response #{response.inspect}"
        raise("Receive Payment Failed with reason #{response.body}") if response.code != 204
      end

      #Order items [{:product => product, :quantity => quantity}]
      def create_and_approve_book_sales_order!(order_items, address={})
        address = DEFAULT_ADDRESS.merge(address)
        customer  = create_customer!(address[:first_name], address[:second_name], address[:line1], address[:line2], address[:city], address[:state], address[:pincode])
        order     = create_book_sales_order!(customer, order_items)
        create_and_approve_payment(order, customer)
        response = oms_client.get_order_by_id(order.id)
        assert_equal("approved", JSON(response.body)["status"],"Order #{order.id} status")
        return customer, order, order.order_items.collect{|order_item| order_item.id}
      end

      def findOrders(params="")
        oms_client.findOrders(params)
      end

      def cancel_order(order_id, change_reason="cancellation_requested", change_sub_reason="customer_requested", refund_type="CREDIT_CARD")
        cancel_order_body = {
            :change_reason => change_reason,
            :change_sub_reason => change_sub_reason,
            :change_data => {
                :refund_type => refund_type
            }
        }
        oms_client.cancel_order(order_id, cancel_order_body.to_json)
      end

      def cancel_order_item(order_item_id, quantity, change_reason="cancellation_requested", change_sub_reason="customer_requested", refund_type="CREDIT_CARD")
        cancel_order_body = {
            :quantity => quantity,
            :change_reason => change_reason,
            :change_sub_reason => change_sub_reason,
            :change_data => {
                :refund_type => refund_type
            }
        }
        oms_client.cancel_order_item(order_item_id, cancel_order_body.to_json)
      end

    end
  end
end
