require 'json'
#Mixin of OMSOperations requires
#   : oms_client function implementation in the class which is mixin OMSOperations
#   : customer_client function implementation in the class which is mixin OMSOperations
#   : shelves function to return hash of shelf hashies
module SC
  module Operations
    module ProcOperations
      def create_requirement(request, headers={:content_type => :json})
        proc_client.create_requirement(request, headers)
      end

      def get_requirements()
        proc_client.get_requirements
      end

      def get_requirements!()
        response = get_requirements
        raise("Get Requirements Failed with reason #{response.body}") if response.status != 200
      end

      def search_requirements(query_params,headers={:content_type => :json})
        proc_client.search_requirements(query_params,headers)
      end

      def approve_requirements(ids, headers={:content_type => :json})
        proc_client.approve_requirements(ids, headers)
      end

      def approve_requirements!(ids)
        response = approve_requirements(ids)
        raise("Approve Requirements failed with reason #{response.body}") if response.code != 204
        response
      end

      def create_purchase_order(request, headers={:content_type => :json})
        proc_client.create_purchase_order(request, headers)
      end

      def create_purchase_order!(request, headers={:content_type => :json})
        response = create_purchase_order(request)
        raise("PO Creation failed with reason #{response.body}") if response.code != 201
        response
      end

      def approve_requirements_and_create_purchase_order(requirement_ids)
        approve_requirements!(requirement_ids.join(","))
        response = create_purchase_order!({:requirement_id_list => requirement_ids}.to_json)
        return Hashie::Mash.new(JSON(response.body)), JSON(response.body)["purchase_orders"].collect {|purchase_order_json| purchase_order_json["id"]}
      end

      def receive_against_purchase_order(id, request, headers={:contenty_type => :json})
        proc_client.receive_against_purchase_order(id, request, headers)
      end

      def receive_against_purchase_order!(id, request)
        response = receive_against_purchase_order(id, request)
        raise("Receive Against PO failed with reason #{response.body}") if response.code != 200
        return response
      end

      def get_purchase_orders(ids, headers={:content_type => :json})
        proc_client.get_purchase_orders(ids, headers)
      end

      def get_purchase_orders!(ids, headers={:content_type => :json})
        response = get_purchase_orders(ids, headers)
        raise("Get purchase orders failed with reason #{response.body}") if response.code != 200
        response
      end

      def get_purchase_order_item(po_id, po_item_id, headers={:content_type => :json})
        proc_client.get_purchase_order_item(po_id, po_item_id, headers)
      end

      def get_purchase_order_item!(po_id, po_item_id)
        response = get_purchase_order_item(po_id, po_item_id)
        raise("Get purchase order item failed with reason #{response.body}") if response.code != 200
        response
      end

      def search_purchase_order_items(params={})
        proc_client.search_purchase_order_items(params)
      end

      def find_po(params={})
        proc_client.find_po(params)
      end

      def search_po_and_po_items_for(ids, headers={:content_type => :json})
        proc_client.search_po_and_po_items_for(ids, headers)
      end
    end
  end
end
