require_relative '../constants/http_status_codes'
require_relative '../factory/warehouse_factory'
require File.expand_path(File.dirname(__FILE__) + '/../utils/logger.rb')
require "active_support/all"
#Mixin of WarehouseOperations requires
#   : warehouse_client function implementation in the class which is mixin WarehouseOperations
#   : shelves function to return hash of shelf hashies
#   : store_locations function to return hash of shelf hashies

module SC
  module Operations
    module WarehouseOperations
      include SC::Factory
      include SC::Constants::HttpStatusCodes
      include SC::TestCore::Logger
      FORWARD_AREA = "FORWARD_AREA"
      STORE = "STORE"
      PENDING_RESERVATION_DEFAULT_ATTRIBUTES = {:ship_by_date => 2.days.from_now}
      RESERVATION_DEFAULT_ATTRIBUTES = {:ship_by_date => 2.days.from_now}

      #products to get atp
      #[
      #    {
      #        :fsn => @product1.fsn,
      #        :sku => @product1.sku
      #    }
      #]
      def get_atp (products)
        warehouse_client.get_ATP({:products => products})
      end

      #Create Pending Reservation using Service
      def create_pending_reservation(warehouse_id, order_item_id, product, quantity, address_id)
        create_bulk_pending_reservation([{:warehouse_id => warehouse_id,
                                          :order_item_id => order_item_id,
                                          :fsn => product.fsn,
                                          :sku => product.sku,
                                          :quantity => quantity,
                                          :address_id => address_id}])[0]
      end

      #Create Bulk Pending Reservation using Service
      # pending_reservations
      #[{:warehouse_id => warehouse_id,
      #  :order_item_id => order_item_id,
      #  :fsn => product.fsn,
      #  :sku => product.sku,
      #  :quantity => quantity,
      #  :address_id => address_id}]
      def create_bulk_pending_reservation(pending_reservations)
        pending_reservation_requests  = pending_reservations.map do |pending_reservation|
          PENDING_RESERVATION_DEFAULT_ATTRIBUTES.merge(pending_reservation)
        end
        request_body = {:pending_reservations => pending_reservation_requests}
        return warehouse_client.create_bulk_pending_reservations(request_body), JSON.parse(request_body.to_json)
      end

      # pending_reservations
      #[{:warehouse_id => warehouse_id,
      #  :order_item_id => order_item_id,
      #  :fsn => product.fsn,
      #  :sku => product.sku,
      #  :quantity => quantity,
      #  :address_id => address_id}]
      def create_bulk_pending_reservation!(pending_reservations)
        response,request_json =  create_bulk_pending_reservation(pending_reservations)
        raise("Create bulk pending reservation failed with reason  #{JSON.parse(response.body)}") if response.code != STATUS_CREATED
        return response, request_json
      end

      def get_pending_reservations_by_order_id(order_id)
        response = warehouse_client.get_pending_reservations_by_order_id(order_id)
        return response, JSON(response.body)
      end

      def get_pending_reservations_by_order_id!(order_id)
        response, response_json = get_pending_reservations_by_order_id(order_id)
        raise("Get Pending Reservation failed with reason  #{JSON.parse(response.body)}") if response.code != STATUS_GET
        return response, response_json
      end

      #Add to inventory using service
      def add_to_inventory(warehouse_id, product, quantity, location_type, po_id =nil)
        request_body =     WarehouseFactory.create_add_to_inventory_body(product.fsn,
                                                                         product.sku,
                                                                         quantity,
                                                                         location_type,
                                                                         po_id)

        warehouse_client.add_to_inventory(warehouse_id, request_body)
      end

      def add_to_inventory!(warehouse_id, product, quantity, location_type, po_id =nil)
        response = add_to_inventory(warehouse_id, product, quantity, location_type, po_id)
        raise("Add To Inventory failed with error #{response.body}") if response.code != STATUS_CREATED
        return response
      end

      #reservations
      #[{:order_item_id => order_item_id,
      #  :quantity => quantity,
      #  :fsn => product.fsn ,
      #  :sku => product.sku,
      #  :warehouse_id => warehouse_id,
      #  :address_id => address_id}]
      def create_bulk_reservation(reservations)
        reservation_requests  = reservations.map do |pending_reservation|
          RESERVATION_DEFAULT_ATTRIBUTES.merge(pending_reservation)
        end
        request_body = {:reservations => reservation_requests}
        return warehouse_client.create_bulk_reservations(request_body), JSON.parse(request_body.to_json)
      end

      #Assing Shelf using Service
      def assign_shelf(warehouse_id, product, source_location_name, destination_label_name, quantity)
        request_body = WarehouseFactory.create_assign_shelf_body(product.fsn,
                                                                 product.sku,
                                                                 source_location_name,
                                                                 destination_label_name,
                                                                 quantity)
        warehouse_client.assign_shelf(warehouse_id, request_body)
      end

      #Assing Shelf using Service
      def assign_shelf!(warehouse_id, product, source_location_name, destination_label_name, quantity)
        response = assign_shelf(warehouse_id, product, source_location_name, destination_label_name, quantity)
        raise("Assign Shelf Failed with reason #{response.body}") if response.code != STATUS_CREATED
        return response, JSON(response.body)["assigned_shelves"][0]["shelf"]
      end

      #Result is same as what you see on create picklist page, when you land on it
      def get_pickables(warehouse_id, location_type, processing_method = "#{location_type}_picklist")
        warehouse_client.get_pickables(warehouse_id, location_type, processing_method)
      end

      def get_pickables!(warehouse_id, location_type, processing_method = "#{location_type}_picklist")
        response = get_pickables(warehouse_id, location_type, processing_method)
        raise("Get Pickables Failed with reason #{response.body}") if response.code != STATUS_GET
        return response
      end

      def get_fa_pickables(warehouse_id)
        warehouse_client.get_pickables(warehouse_id, FORWARD_AREA)
      end

      def get_store_pickables(warehouse_id)
        warehouse_client.get_pickables(warehouse_id, STORE)
      end

      #create pick list using Service
      def create_transfer_list(warehouse_id, picking_zone_id, count = nil)
        create_picklist_payload = WarehouseFactory.create_create_transfer_list_body(picking_zone_id,
                                                                               count)

        warehouse_client.create_transfer_list(warehouse_id,create_picklist_payload)
      end

      #create pick list using Service
      def create_picklist(warehouse_id, area, picklist_type, create_picklist_count = nil)
        create_picklist_payload = WarehouseFactory.create_create_picklist_body(area,
                                                                               picklist_type,
                                                                               create_picklist_count)

        warehouse_client.create_picklist(warehouse_id,create_picklist_payload)
      end

      #create pick list using Service
      def create_picklist!(warehouse_id, area, picklist_type, create_picklist_count = nil)
        response = create_picklist(warehouse_id, area, picklist_type, create_picklist_count)
        raise("Create Picklist Failed with reason #{response.body}") if (response.code != STATUS_CREATED and response.code != STATUS_GET)
        response
      end

      def get_picklist(warehouse_id, picklist_id)
        warehouse_client.get_picklist(warehouse_id, picklist_id)
      end

      def pick_picklist(warehouse_id, picklist_id)
        warehouse_client.pick_picklist(warehouse_id, picklist_id)
      end

      def pick_picklist_item(warehouse_id, picklist_id, picklist_item_id, quantity)
        warehouse_client.pick_picklist_item(warehouse_id, picklist_id, picklist_item_id, quantity)
      end

      def pick_picklist!(warehouse_id, picklist_id)
        response = pick_picklist(warehouse_id, picklist_id)
        raise("Pick Picklist Failed with Response #{JSON.parse(response.body)}") if response.code != STATUS_NO_CONTENT
        return response
      end

      #putlist_items
      #[
      #    {
      #        :fsn => @product1.fsn,
      #        :sku => @product1.sku,
      #        :quantity => 2
      #    }
      #]
      def create_putlist(warehouse_id, location_type, putlist_items)
        request_body = {:putlist_items => putlist_items, :location => location_type}
        return warehouse_client.create_putlist(warehouse_id, request_body), JSON.parse(request_body.to_json)
      end

      #putlist_items
      #[
      #    {
      #        :fsn => @product1.fsn,
      #        :sku => @product1.sku,
      #        :quantity => 2
      #    }
      #]
      def update_putlist(warehouse_id, putlist_id, putlist_items)
        request_body = {:putlist_items => putlist_items}
        return warehouse_client.update_putlist(warehouse_id, putlist_id, request_body), JSON.parse(request_body.to_json)
      end

      #putlist_items
      #[
      #    {
      #        :fsn => @product1.fsn,
      #        :sku => @product1.sku,
      #        :quantity => 2
      #    }
      #]
      def update_putlist!(warehouse_id, putlist_id, putlist_items)
        response, response_json = update_putlist(warehouse_id, putlist_id, putlist_items)
        raise("Update Putlist Failed with Response #{JSON.parse(response.body)}") if response.code != STATUS_GET
        return response, response_json
      end

      def get_putlist(warehouse_id, putlist_id, suggestion = nil)
        warehouse_client.get_putlist(warehouse_id, putlist_id, suggestion)
      end

      def get_putlist_item(warehouse_id, putlist_id, putlist_item_id)
        warehouse_client.get_putlist_item(warehouse_id, putlist_id,putlist_item_id)
      end

      #cancellations
      #[
      #    {
      #        :warehouse_id => @del_warehouse.id,
      #        :max_quantity => 6,
      #        :address_id => 1
      #    }
      #]
      def cancel_order_item(order_item_id, cancellations)
        warehouse_client.cancel_order_item(order_item_id, {:cancellations => cancellations})
      end

      #address_ids comma separated address ids
      def get_order_item_status(order_item_id, address_ids_str)
        warehouse_client.get_order_item_status(order_item_id, address_ids_str)
      end

      #Create Random FA Reservation
      def create_FA_reservations(warehouse_id, how_many_reservations, use_random_quantity=true, max_quantity=2)
        # Create Multiple Reservations
        reservations_to_create = []
        how_many_reservations.times do |i|
          product   = get_random_product
          quantity  = max_quantity
          quantity  = (rand(max_quantity) + 1) if use_random_quantity
          order_item_id  = Time.new.to_i + i
          reservations_to_create << {
              :product => product,
              :quantity => quantity,
              :order_item_id => order_item_id,
              :address_id => (i+1),
          }
        end
        create_FA_reservations_with_data(warehouse_id, reservations_to_create)
      end

      # Create FA reservations passed by user
      #reservations_to_create
      #[
      #    {
      #        :product => product,
      #        :quantity => 2,
      #        :order_item_id=> 12345,
      #        :address_id => 1
      #        :location => self OPTIONAL
      #    }
      #]
      def create_FA_reservations_with_data(warehouse_id, reservations_to_create)
        # Create Multiple Reservations
        index = 1
        reservations_to_create.collect do |reservation_to_create|
          logger.info  "\n==Creating Reservation #{index}=="
          product   = reservation_to_create[:product]
          quantity  = reservation_to_create[:quantity]
          shelf     = reservation_to_create[:location].nil? ? shelves[rand(shelves.size)].label : reservation_to_create[:location]
          order_item_id  = reservation_to_create[:order_item_id]
          address_id  = reservation_to_create[:address_id]
          po_id  = Time.new.to_i

          create_pending_reservation(warehouse_id, order_item_id, product, quantity,address_id)

          add_to_inventory!(warehouse_id, product, quantity, FORWARD_AREA, po_id)

          assign_shelf!(warehouse_id, product, FORWARD_AREA, shelf, quantity)

          index += 1
          {"order_item_id"=> order_item_id,
           "fsn" => product.fsn,
           "sku" => product.sku,
           "location" => shelf,
           "quantity" => quantity}
        end
      end

      # Create reservations in warehouse
      #Assign shelf would be done using ui
      def create_FA_reservations_using_ui(warehouse_id,reservations,assign_shelf_page)
        expected_picklist_items = []

        # Create Multiple Reservations
        reservations.times do |i|
          # Create Pending Reservation
          logger.info  "\n==Creating Reservation #{i}=="
          product   = get_random_product
          quantity  = rand(2) + 1
          shelf     = shelves[rand(shelves.size)]
          order_item_id  = Time.new.to_i + i
          po_id  = "po_id_#{(Time.new.to_i + i)}"

          create_pending_reservation(warehouse_id, order_item_id, product, quantity, (i+1))

          add_to_inventory(warehouse_id, product, quantity, FORWARD_AREA, po_id)

          assign_shelf_input_entries = [{"fsn" => product.fsn, "sku" => product.sku, "quantity" => quantity}]

          assign_shelf_page.assign_shelf(shelf,assign_shelf_input_entries)

          expected_picklist_items << {"fsn" => product.fsn,
                                      "sku" => product.sku,
                                      "location" => shelf,
                                      "quantity" => quantity,}

        end
        return expected_picklist_items
      end

      #entries_to_create_putlist
      #[
      #    {
      #        :product => product,
      #        :quantity => quantity,
      #        :location =>store_location
      #    }
      #]
      #Returns Create Putlist Response, Putlist Id and putlist item ids
      def add_inventory_and_create_putlist!(warehouse_id, entries_to_create_putlist)

        request_for_putlist = entries_to_create_putlist.collect do |entry_to_create_putlist|

          add_inventory_response = add_to_inventory(warehouse_id, entry_to_create_putlist[:product],
                                                    entry_to_create_putlist[:quantity],
                                                    STORE)
          raise("Add To Inventory Failed with Response #{JSON(add_inventory_response.body)}") if add_inventory_response.code != STATUS_CREATED

          {:fsn => entry_to_create_putlist[:product].fsn,
           :sku => entry_to_create_putlist[:product].sku,
           :quantity => entry_to_create_putlist[:quantity]}
        end

        create_putlist_resp = create_putlist(warehouse_id, 'inward_bulk', request_for_putlist)[0]
        raise("Create Putlist Failed with Response #{JSON(create_putlist_resp.body)}") if create_putlist_resp.code != STATUS_CREATED

        return create_putlist_resp,
            JSON(create_putlist_resp.body)["id"],
            JSON(create_putlist_resp.body)["putlist_items"].collect {|putlist_item| putlist_item["id"]}
      end

      #entries
      #[
      #    {
      #        :product => product,
      #        :quantity => quantity,
      #        :location =>store_location
      #    }
      #]
      #Returns update_putlist response and json body
      def add_inventory_then_create_and_complete_putlist!(warehouse_id, entries)

        entries_to_create_putlist = entries.collect { |entry_to_create_putlist| {:product => entry_to_create_putlist[:product],
                                                                                                   :quantity => entry_to_create_putlist[:quantity]}}

        create_putlist_response, putlist_id, putlist_item_ids = add_inventory_and_create_putlist!(warehouse_id,
                                                                                                                 entries_to_create_putlist)

        entries_to_update_putlist = []
        putlist_item_ids.size.times do |index|
          entries_to_update_putlist << {:id => putlist_item_ids[index],
                                        :quantity_moved => entries[index][:quantity],
                                        :location => entries[index][:location]}
        end
        logger.info entries_to_update_putlist
        update_putlist!(warehouse_id,
                        putlist_id,
                        entries_to_update_putlist)[0]
      end

      #Creates Random Store Reservations
      def create_store_reservations(warehouse_id, how_many_reservations, use_random_quantity=true, max_quantity=2)
        # Create Multiple Reservations
        reservations_to_create = []
        how_many_reservations.times do |i|
          product   = get_random_product
          quantity  = max_quantity
          quantity  = (rand(max_quantity) + 1) if use_random_quantity
          order_item_id  = Time.new.to_i + i
          reservations_to_create << {
              :product => product,
              :quantity => quantity,
              :order_item_id => order_item_id,
              :address_id => (i+1),
          }
        end
        create_store_reservations_with_data(warehouse_id, reservations_to_create)
      end

      # Create store reservations passed by user
      #reservations_to_create
      #[
      #    {
      #        :product => product,
      #        :quantity => 2,
      #        :order_item_id=> 12345,
      #        :address_id => 1,
      #        :location => store location OPTIONAL
      #    }
      #]
      def create_store_reservations_with_data(warehouse_id, reservations_to_create)
        # Create Multiple Reservations
        index = 1
        reservations_to_create.collect do |reservation_to_create|
          logger.info  "\n==Creating Reservation #{index}=="
          product = reservation_to_create[:product]
          quantity = reservation_to_create[:quantity]
          warehouse_store_locations = store_locations



          store_location  = reservation_to_create[:location].nil? ? store_locations.keys[rand(store_locations.keys.size)] : reservation_to_create[:location]
          order_item_id   = reservation_to_create[:order_item_id]
          address_id      = reservation_to_create[:address_id]
          po_id  = Time.new.to_i

          add_inventory_then_create_and_complete_putlist!(warehouse_id,
                                                          [{:product => product, :quantity => quantity, :location =>store_location}])

          create_reservation_response, create_reservation_request_json  = create_bulk_reservation([{:order_item_id => order_item_id,
                                                                                                    :quantity => quantity,
                                                                                                    :fsn => product.fsn ,
                                                                                                    :sku => product.sku,
                                                                                                    :warehouse_id => warehouse_id,
                                                                                                    :address_id => address_id}])

          index += 1
          {"order_item_id"=> order_item_id,
           "fsn" => product.fsn,
           "sku" => product.sku,
           "location" => store_location,
           "quantity" => quantity}
        end
      end

      #Create Random Reservations and Picklists in giver Area. Store/forward_Area
      def create_picklists_with_random_data!(warehouse_id, how_many_reservations, picklist_type, packing_area)

        expected_picklist_items = packing_area.upcase == FORWARD_AREA ?
            create_FA_reservations(warehouse_id, how_many_reservations, false, 2) :
            create_store_reservations(warehouse_id, how_many_reservations, false, 2)

        create_picklist_resp = create_picklist!(warehouse_id,
                                                packing_area,
                                                picklist_type)

        return expected_picklist_items,
            create_picklist_resp,
            JSON(create_picklist_resp.body)["picklists"].collect {|picklist_json| picklist_json["id"]}

      end

      #reservations
      #[
      #    {
      #        :product => product,
      #        :quantity => 2,
      #        :order_item_id=> 12345,
      #        :address_id => 1,
      #        :location => store location/shelf OPTIONAL
      #    }
      #]
      def create_picklists_with_data!(warehouse_id, reservations, picklist_type, packing_area)
        expected_picklist_items = packing_area.upcase == FORWARD_AREA ?
            create_FA_reservations_with_data(warehouse_id, reservations) :
            create_store_reservations_with_data(warehouse_id, reservations)

        create_picklist_resp = create_picklist!(warehouse_id,
                                                packing_area,
                                                picklist_type)

        return expected_picklist_items,
            create_picklist_resp,
            JSON(create_picklist_resp.body)["picklists"].collect {|picklist_json| picklist_json["id"]}
      end

      def reprocess_shipment(shipment_id)
        response = warehouse_client.reprocess_shipment(shipment_id)
        reprocess_shipment_resp_json = JSON(response.body)
        return response, reprocess_shipment_resp_json.delete("new_shipment_id")
      end

      def reprocess_shipment!(shipment_id)
        response, new_shipment_id = reprocess_shipment(shipment_id)
        raise("Reprocess API failed with reason  #{JSON.parse(response.body)}") if response.code != STATUS_GET
        return response, new_shipment_id
      end

      def confirm_shipment_via_warehouse(shipment_id, tracking_id, weight)
        warehouse_client.confirm_shipment(shipment_id, {:weight => weight, :tracking_id => tracking_id})
      end

      def confirm_shipment_via_warehouse!(shipment_id, tracking_id, weight)
        response = warehouse_client.confirm_shipment(shipment_id, {:weight => weight, :tracking_id => tracking_id})
        raise("Confirm Shipment via Warehouse failed with reason  #{JSON.parse(response.body)}") if response.code != STATUS_GET
        response
      end
    end
  end
end