
require 'amqp'
require 'sc_mq'
require 'rack_request'
require 'sc_mq/client/mq_client'
require 'sc_mq/processor/event_machine_runner'
require 'sc_mq/exceptions/mq_exception'
require 'sc_mq/processor/util'
require 'sc_mq/processor/message'
require 'sc_mq/processor/amqp_integration_manager'
require 'sc_mq/processor/consumer_registration_handler'
require 'sc_mq/processor/initial_connection_failure_handlers'
require 'sc_mq/processor/connection_interruption_handlers'
require 'sc_mq/processor/rest_api_handler'
require 'sc_mq/processor/retry_handler'
require 'sc_mq/processor/inbound_message_handler'
require 'sc_mq/processor/outbound_message_handler'
require 'sc_mq/processor/receiver'
require 'sc_mq/processor/relayer'
require 'sc_mq/rack/util'
require 'sc_mq/rack/request'
require 'sc_mq/rack/filter'
require 'sc_mq/inbound_message'
require 'sc_mq/outbound_message'


module ScCore

  class MqConfig
    
    def self.mq_enabled?
      RUBY_PLATFORM == "java"
    end

    @@receiver_connection_settings = {}
    @@relayer_connection_settings = {}
    @@daemons = []
    @@receiver = nil

    if mq_enabled?
      @@receiver_connection_settings = {
          :host => SupplyChain.mq_host,
          :port => 5672,
          :user => "guest",
          :pass => "guest",
          :vhost => "/",
          :heartbeat => 30000,
          :ssl => false
      }
      @@relayer_connection_settings = {
          :host => SupplyChain.mq_host,
          :port => 5672,
          :user => "guest",
          :pass => "guest",
          :vhost => "/",
          :heartbeat => 30000,
          :ssl => false
      }
      sc_receive_from_queue = SupplyChain.queue_name
      relayer_batch_size = 100
      @@receiver = ScMq::Processor::Receiver.new(SupplyChain.app_id, @@receiver_connection_settings, sc_receive_from_queue)
      @@relayer = ScMq::Processor::Relayer.new(@@relayer_connection_settings, relayer_batch_size)
      @@daemons = [@@receiver, @@relayer]
      logger.info "Initialized daemons #{@@daemons.inspect}."
    else
      logger.warn "Not initializing MQ daemons as patform is #{RUBY_PLATFORM}"
    end
    
    def self.daemons_to_run
      @@daemons
    end

    def self.receiver
      @@receiver
    end

    def self.receiver_connection_settings
      @@receiver_connection_settings
    end

    def self.relayer_connection_settings
      @@relayer_connection_settings
    end

  end

end
