$:.unshift File.expand_path(File.dirname(__FILE__) + '/../lib')

ENV["DEPLOY_ENV"] = "staging"
ENV["PADRINO_ENV"] = "test"
ENV["SUPPLY_CHAIN_COMPANY"] = "b2b"

module FloConfig
  CONFIG = {}
end

SERVICE_NAME = 'warehouse'

require 'test/unit'
require 'sc_core/supply_chain'

class SupplyChainTest < Test::Unit::TestCase

  def setup
    FloConfig::CONFIG[:staging] = { :test => { } }
  end

  def test_urls
    assert_equal "http://staging.b2b.fulfillment.flipkart.com:4301", SupplyChain.urls["fulfillment_url"]
    assert_equal "http://staging.b2b.fulfillment.flipkart.com:4301", SupplyChain.urls["fulfillment_b2b_url"]
    assert_equal "http://staging.b2c.fulfillment.flipkart.com:3301", SupplyChain.urls["fulfillment_b2c_url"]
  end

  def test_cache_config_with_defaults
    FloConfig::CONFIG[:staging][:test][:cache] = { :urls => ["stage-cache.nm.flipkart.com:11211"] }

    expected_config = {
      :urls=>["stage-cache.nm.flipkart.com:11211"],
      :bucket => "flo_warehouse_b2b_staging",
      :username => "flo_warehouse_b2b_staging",
      :password => "",
      :namespace => ""
    }

    assert_equal expected_config, SupplyChain.cache_config
  end

  def test_cache_config_with_overriden_keys
    FloConfig::CONFIG[:staging][:test][:cache] = {
      :urls => ["stage-cache.nm.flipkart.com:11211"],
      :username => 'me',
      :password => 'password',
      :bucket => 'some_bucket',
      :namespace => 'some_namespace'
    }

    expected_config = {
      :urls=>["stage-cache.nm.flipkart.com:11211"],
      :bucket => "flo_warehouse_b2b_staging", # :bucket is not overridable with FloConfig
      :username => "me",
      :password => "password",
      :namespace => "some_namespace"
    }

    assert_equal expected_config, SupplyChain.cache_config
  end

  def test_cache_config_should_throw_error_for_missing_flo_config
    FloConfig::CONFIG[:staging][:test][:cache] = nil
    assert_raises(RuntimeError, 'Missing :cache config in flo/env_config.rb') do
      SupplyChain.cache_config
    end
  end

  def test_cache_bucket
    assert_equal 'flo_warehouse_b2c_staging', SupplyChain.cache_bucket('warehouse', 'b2c')
    assert_equal 'flo_oms_b2b_staging', SupplyChain.cache_bucket('oms', 'b2b')
  end
end
