require_relative 'sc_client.rb'

module SC
  module Client
    class PaymentClient
      @@PAYMENT_ID = "PAYMENT_ID"
      @@PARAM_STRING = "PARAM_STRING"
      @@ACTION_ID = "ACTION_ID"
      @@INVOICE_ID = "INVOICE_ID"
      @@CREATE_PAYMENT_RESOURCE = "/payments"
      @@GET_PAYMENTS_RESOURCE = "/payments?"  + @@PARAM_STRING
      @@GET_PAYMENT_RESOURCE =  "/payments/" + @@PAYMENT_ID
      @@DO_ACTION_ON_PAYMENT_RESOURCE = "/payments/" + @@PAYMENT_ID + "/" + @@ACTION_ID
      @@CREATE_INVOICE_RESOURCE = "/invoices"
      @@GET_INVOICE_RESOURCE = "/invoices/" + @@INVOICE_ID
      @@PRINT_INVOICE_RESOURCE = "/invoices/documents"
      @@DO_ACTION_ON_INVOICE_RESOURCE = "/invoices/" + @@INVOICE_ID + "/" + @@ACTION_ID

      def initialize(client)
        @client = client
      end

      def add_payment(payload, headers={:content_type => :json})
        @client.post(@@CREATE_PAYMENT_RESOURCE, payload, headers)
      end

      def get_payments(params, headers={:content_type => :json})
        resource_location = @@GET_PAYMENTS_RESOURCE.sub(@@PARAM_STRING, params)
        @client.get(resource_location, headers)
      end

      def get_payment_by_id(payment_id, headers={:content_type => :json})
        resource_location = @@GET_PAYMENT_RESOURCE.sub(@@PAYMENT_ID, payment_id.to_s)
        @client.get(resource_location, headers)
      end

      def do_action_on_payment_id(payment_id, action, payload, headers={:content_type => :json})
        resource_location =  @@DO_ACTION_ON_PAYMENT_RESOURCE.sub(@@PAYMENT_ID, payment_id.to_s).sub(@@ACTION_ID, action.to_s)
        @client.put(resource_location, payload, headers)
      end

      def create_invoice(payload, headers={:content_type => :json})
        @client.post(@@CREATE_INVOICE_RESOURCE, payload, headers)
      end

      def get_invoice_by_id(invoice_id, headers={:content_type => :json})
        resource_location = @@GET_INVOICE_RESOURCE.sub(@@INVOICE_ID, invoice_id.to_s)
        @client.get(resource_location, headers)
      end

      def print_invoice(payload, headers={:content_type => :json})
        @client.post(@@PRINT_INVOICE_RESOURCE, payload, headers)
      end

      def do_action_on_invoice_id(invoice_id, action, payload="", headers={:content_type => :json})
        resource_location =  @@DO_ACTION_ON_INVOICE_RESOURCE.sub(@@INVOICE_ID, invoice_id.to_s).sub(@@ACTION_ID, action.to_s)
        @client.put(resource_location, payload, headers)
      end

    end
  end
end
