module SC
  module Client
    class ShippingClient
      @@SHIPMENT_ID_TEMP  = "-SHIPMENT_ID-"
      @@CREATE_RESOURCE   = "/shipments"
      @@CONFIRM_RESOURCE  = "/shipments/"+@@SHIPMENT_ID_TEMP+"/confirm"
      @@CANCEL_RESOURCE  = "/shipments/"+@@SHIPMENT_ID_TEMP+"/cancel"
      @@GET_RESOURCE      = "/shipments/"+@@SHIPMENT_ID_TEMP
      @@GET_SHIPMENT_ITEM_RESOURCE = "/shipments/"+@@SHIPMENT_ID_TEMP

      def initialize(client)
        @client = client
      end

      #Shipping Operations
      def create_shipment(json_shipment_payload, headers={:content_type => :json})
        puts "\nCreating Shipment with payload #{json_shipment_payload.to_json}"
        response  = @client.post(@@CREATE_RESOURCE,json_shipment_payload.to_json, headers)
        puts "Create Shipment Response #{response.inspect}"
        return response
      end

      def confirm_shipment(shipment_id, json_confirm_shipment_payload, headers={:content_type => :json})
        puts "\nConfirming Shipment #{shipment_id} with payload #{json_confirm_shipment_payload} "
        response =  @client.post(@@CONFIRM_RESOURCE.sub(@@SHIPMENT_ID_TEMP,shipment_id.to_s),json_confirm_shipment_payload.to_json, headers)
        puts "Confirm Shipment Response #{response.inspect}"
        return response
      end

      def cancel_shipment(shipment_id, headers={:content_type => :json})
        puts "\nCancelling #{shipment_id} "
        response =  @client.post(@@CANCEL_RESOURCE.sub(@@SHIPMENT_ID_TEMP,shipment_id.to_s),{}, headers)
        puts "Cancel Shipment Response #{response.inspect}"
        return response
      end

      def get_shipment(shipment_id)
        puts "\nGetting Shipment #{shipment_id}"
        response  = @client.get(@@GET_RESOURCE.sub(@@SHIPMENT_ID_TEMP,shipment_id.to_s), headers={})
        puts "Get Shipment Response #{response.inspect}"
        return response
      end

      def get_shipment_items_status(order_item_id, address_ids_str)
        resource = "/shipment_items?order_item_id=#{order_item_id}&address_ids=#{address_ids_str}"
        puts "\nGetting Shipment items status using resource #{resource}"
        response  = @client.get(resource, headers={})
        puts "Get Shipment Items Status Response #{response.inspect}"
        return response
      end

      def delete_shipment(shipment_id)
        puts "Deleting Shipment #{shipment_id}"
        response @client.delete(@@DELETE_RESOURCE.sub(@@SHIPMENT_ID_TEMP,shipment_id.to_s), headers={})
        puts "Delete Shipment Response #{response.inspect}"
        return response
      end

    end
  end
end