require 'logger'

module Rack
  # Releases ActiveRecord connections checked out by the current thread.
  class LogExceptions
    def initialize(app)
      @app = app
    end

    def call(env)
      @app.call env
    rescue
      logger.error "Exception caught #{$!}"
      logger.error $!.backtrace.join("\n")
      [500, {'Content-Type' => 'text/plain'}, ['']]
    end

    private
    def logger
      if defined?(Padrino)
        Padrino.logger
      elsif defined?(Rails)
        Rails.logger
      else
        Logger.new(STDOUT)
      end
    end
  end
end
