require 'sc-rest-client/rest_helpers'

module Rack
  # Sets up thread-locals to propagate transaction-ids
  class TransactionTracer
    include RestClient::RestHelpers

    def initialize(app)
      @app = app
    end

    def call(env)
      txn_id ||= env["HTTP_X_RESTBUS_TRANSACTION_ID"] if env["HTTP_X_RESTBUS_TRANSACTION_ID"].present?
      txn_id ||= env["HTTP_X_TRANSACTION_ID"] if env["HTTP_X_TRANSACTION_ID"].present?
      txn_id ||= generate_transaction_id
      set_transaction_id txn_id
      env["HTTP_X_RESTBUS_TRANSACTION_ID"] = env["X_RESTBUS_TRANSACTION_ID"] = txn_id
      env["HTTP_X_TRANSACTION_ID"] = env["X_TRANSACTION_ID"] = txn_id

      set_request_id(req_id = generate_request_id)
      env["HTTP_X_REQUEST_ID"] = env["X_REQUEST_ID"] = req_id

      @app.call(env)
    ensure
      reset_transaction_id
      reset_request_id
    end
  end
end