require_relative '../utils/rest_client_wrapper'
module SC
  module Utils
    class PlatformQueueClient
      attr_reader :base_url, :rest_client

      STATUS_NO_CREATION  = 200
      STATUS_CREATED      = 201
      STATUS_UPDATED      = 200
      STATUS_DELETED      = 200
      STATUS_GET          = 200
      STATUS_NOT_FOUND    = 404
      STATUS_BAD_REQUEST  = 400
      STATUS_NO_CONTENT   = 204

      def initialize(base_url, username, password)
        @base_url     = base_url.sub("http://","http://#{username}:#{password}@")
        puts "Base URL '#{@base_url}'"
        @rest_client  = RestClientWrapper.new @base_url
      end

      def get_messages(queue_name, how_many_messages=1, headers={:content_type => :json})
        resource_url  = "/api/queues/%2F/#{queue_name}/get"
        payload = {
            :vhost => "/",
            :name =>"ci.b2c.oms.fanout.queue",
            :truncate =>"50000",
            :requeue =>"true",
            :encoding =>"auto",
            :count =>"#{how_many_messages}"
        }

        puts "\nGetting Queue messages using resource #{resource_url} and payload #{payload}"
        response  = rest_client.post_json(resource_url ,
                                          payload, headers)

        puts "Get Queue Messages Response #{response}"
        return response
      end

      def purge(queue_name)
        resource_url  = "/api/queues/%2F/#{queue_name}/contents"
        puts "\nPurgin Queue messages using resource #{resource_url}"
        response  = rest_client.delete(resource_url ,{})

        puts "Purge Queue Messages Response #{response}"
        return response
      end
    end
  end
end
