HealthApp.helpers do
  SUPPLY_CHAIN_COMPANY ||= ENV["SUPPLY_CHAIN_COMPANY"]

  APP_CHECKS_TEMPLATE = [
      {#Just checks the socket connection to port
       :name => "app_check",
       :check_type => :socket,
       :host => :app_host,
       :port => :app_port
      }
  ]

  DB_CHECK_TEMPLATE = [
      {#Connect to db with given db
       :name => "db_check",
       :check_type => :mysql,
       :db_host => :db_host,
       :db_port => 3601,
       :user_name => "root",
       :password => "",
       :db => :db
      }
  ]

  QUEUE_CHECK_TEMPLATE = [
      {
          :name => "Queue Connection",
          :check_type => :queue_connection,
          :queue_host => "erp-deb-ci-vm4.corp.flipkart.com",
          :queue_port => 55672,
          :queue_name => :queue_name,
          :user_name => "guest",
          :password => "guest"
      },
      {
          :name => "Queue Consumer Connection Break",
          :check_type => :queue_consumer_check,
          :queue_host => "erp-deb-ci-vm4.corp.flipkart.com",
          :queue_port => 55672,
          :queue_name => :queue_name,
          :user_name => "guest",
          :password => "guest",
          :max_consumers => 1,
          :min_consumers => 0,
      }
  ]

  def instantiate_checks_template(checks_template, input)
    checks = checks_template.collect do |check_template|
      check = check_template.clone
      check.each do |key, value|
        check[key] = input[value] if input.keys.include?(value)
      end
      check
    end
  end

  def get_app_checks(suppy_chain_company, service)
    input = {

        :app_host => "#{PADRINO_ENV}.#{suppy_chain_company}.#{service}.flipkart.com",
        :app_port => SupplyChain::SERVICES[service.to_sym][SUPPLY_CHAIN_COMPANY.to_sym]
    }
    instantiate_checks_template(APP_CHECKS_TEMPLATE, input)
  end

  def get_db_checks(suppy_chain_company, service)
    input = {
        :db_host => "#{PADRINO_ENV}.#{suppy_chain_company}.#{service}.db.flipkart.com",
        :db => "#{service}_#{suppy_chain_company}_#{PADRINO_ENV}"
    }

    instantiate_checks_template(DB_CHECK_TEMPLATE, input)
  end

  def get_queue_checks(suppy_chain_company, service)
    input = {
        :queue_name => "#{service}_#{suppy_chain_company}_#{PADRINO_ENV}.fanout.queue"
    }

    instantiate_checks_template(QUEUE_CHECK_TEMPLATE, input)
  end

  def check_socket_connection(host, port)
    s = TCPSocket.open(host, port)
    s.close # Close the socket when done
    return true, ""
  rescue => e
    return false, "Socket check for host:#{host} and port: #{port} failed with error #{e.message}"
  end

  def check_mysql_connection(host, port, user, password, db)
    #connection = Mysql2::Client.new(:host => "localhost", :port => port, :username => user, :password =>password, :database => db)
    return true, ""
  rescue => e
    return false, "DB #{db} connection failed with error #{e.message}"
  end

  def check_queue_connection(queue_host, queue_port, queue_name)
    url = "http://guest:guest@#{queue_host}:#{queue_port}/api/queues/%2F/#{queue_name}"
    response = RestClient.get(url, {:content_type => :json}) do |response, request, result|
      response
    end
    return response.code == 200, "#{queue_name} Queue Presence Check Failed with with error #{response.body}"
  rescue => e
    return false, "#{queue_name} Queue Presence Check Failed with with error #{e.message}"
  end

  def check_queue_consumers(queue_host, queue_port, queue_name, min_consumers, max_consumers)
    url = "http://guest:guest@#{queue_host}:#{queue_port}/api/queues/%2F/#{queue_name}"
    response = RestClient.get(url, {:content_type => :json}) do |response, request, result|
      response
    end
    consumers = JSON(response.body)["consumers"]
    return consumers >= min_consumers && consumers <= max_consumers, "Current #{queue_name} Queue Consumers : #{consumers}"
  rescue => e
    return false, "#{queue_name} Queue consumer Check Failed with with error #{e.message}"
  end

  def erp_checks
    {
        :oms_b2c => get_app_checks(:b2c, :oms) | get_queue_checks(:b2c, :oms) | get_db_checks(:b2c, :oms),
        :oms_b2b => get_app_checks(:b2b, :oms) | get_queue_checks(:b2b, :oms) | get_db_checks(:b2b, :oms),

        :customer_b2c => get_app_checks(:b2c, :customer) | get_db_checks(:b2c, :customer),
        :customer_b2b => get_app_checks(:b2b, :customer) | get_db_checks(:b2b, :customer),

        :payment_b2c => get_app_checks(:b2c, :payment) | get_db_checks(:b2c, :payment),
        :payment_b2b => get_app_checks(:b2b, :payment) | get_db_checks(:b2b, :payment),

        :fulfillment_b2c => get_app_checks(:b2c, :fulfillment) | get_queue_checks(:b2c, :fulfillment) | get_db_checks(:b2c, :fulfillment),
        :fulfillment_b2b => get_app_checks(:b2b, :fulfillment) | get_queue_checks(:b2b, :fulfillment) | get_db_checks(:b2b, :fulfillment),

        :proc_b2c => get_app_checks(:b2c, :proc) | get_queue_checks(:b2c, :proc) | get_db_checks(:b2c, :proc),
        :proc_b2b => get_app_checks(:b2b, :proc) | get_queue_checks(:b2b, :proc) | get_db_checks(:b2b, :proc),

        :warehouse_b2c => get_app_checks(:b2c, :warehouse) | get_queue_checks(:b2c, :warehouse) | get_db_checks(:b2c, :warehouse),
        :warehouse_b2b => get_app_checks(:b2b, :warehouse) | get_queue_checks(:b2b, :warehouse) | get_db_checks(:b2b, :warehouse),

        :shipping_b2c => get_app_checks(:b2c, :shipping) |  get_db_checks(:b2c, :shipping),
        :shipping_b2b => get_app_checks(:b2b, :shipping) |  get_db_checks(:b2b, :shipping),

        :doc_b2c => get_app_checks(:b2c, :doc) | get_db_checks(:b2c, :doc),
        :doc_b2b => get_app_checks(:b2b, :doc) | get_db_checks(:b2b, :doc),

        :supplier_b2c => get_app_checks(:b2c, :supplier) | get_db_checks(:b2c, :supplier),
        :supplier_b2b => get_app_checks(:b2b, :supplier) | get_db_checks(:b2b, :supplier),

        :geocourier_b2c => get_app_checks(:b2c, :geocourier) | get_db_checks(:b2c, :geocourier),
        :geocourier_b2b => get_app_checks(:b2b, :geocourier) | get_db_checks(:b2b, :geocourier),

        :nss_b2c => get_app_checks(:b2c, :nss) | get_db_checks(:b2c, :nss),
        :nss_b2b => get_app_checks(:b2b, :nss) | get_db_checks(:b2b, :nss),

    }
  end

  def check_service_is_up(name, checks)
    pass = true
    error_msg = ""
    reasons = {}
    checks.each do |check|
      case check[:check_type]
        when :socket
          result, check_msg = check_socket_connection(check[:host], check[:port])
        when :mysql
          result, check_msg = check_mysql_connection(check[:db_host], check[:db_port], check[:user_name], check[:password], check[:db])
        when :queue_connection
          result, check_msg = check_queue_connection(check[:queue_host], check[:queue_port], check[:queue_name])
        when :queue_consumer_check
          result, check_msg = check_queue_consumers(check[:queue_host], check[:queue_port], check[:queue_name], check[:min_consumers], check[:max_consumers])
        else
          result, check_msg = false, "Check '#{check[:check_type]}' not supported"
      end
      pass &= result

      unless result
        reasons[check[:check_type]] = check_msg
      end
    end

    {
        :status => pass,
        :reasons => reasons
    }
  end
end

