$:.unshift File.expand_path(File.dirname(__FILE__) + '/../lib')
require 'test/unit'
require 'active_record'
require 'rack/connection_release'

class ConnectionReleaseTest < Test::Unit::TestCase
  def setup
    ActiveRecord::Base.establish_connection(
        :adapter => "mysql",
        :host => "localhost",
        :username => "root",
        :password => "",
        :database => "mysql"
    )
  end

  def test_should_release_active_connections_after_app_call_is_done
    app = lambda { |env|
      assert ! ActiveRecord::Base.connection_handler.active_connections?
      ActiveRecord::Base.connection
      assert ActiveRecord::Base.connection_handler.active_connections?
      [200, {}, ["I've checked-out a connection."]]
    }

    Rack::ConnectionRelease.new(app).call({})

    assert !ActiveRecord::Base.connection_handler.active_connections?, "connection wasn't released"
  end

  def test_should_release_active_connections_even_if_the_app_raises_an_exception
    app = lambda { |env|
      assert ! ActiveRecord::Base.connection_handler.active_connections?
      ActiveRecord::Base.connection
      assert ActiveRecord::Base.connection_handler.active_connections?
      raise "crashing!"
    }

    assert_raise(RuntimeError) { Rack::ConnectionRelease.new(app).call({}) }

    assert !ActiveRecord::Base.connection_handler.active_connections?, "connection wasn't released"
  end
end
