require_relative 'sc_client.rb'

module SC
  module Client

    class CustomerClient
      @@CUSTOMER_ID = "CUSTOMER_ID"
      @@ADDRESS_ID = "ADDRESS_ID"
      @@PARAM_STRING = "PARAM_STRING"
      @@CREATE_CUSTOMER_RESOURCE = "/customers"
      @@GET_CUSTOMER_RESOURCE =  "/customers/" + @@CUSTOMER_ID
      @@GET_CUSTOMERS_RESOURCE = "/customers?" + @@PARAM_STRING
      @@DELETE_CUSTOMER_RESOURCE = "/customers/" + @@CUSTOMER_ID
      @@UPDATE_CUSTOMER_RESOURCE = "/customers/" + @@CUSTOMER_ID
      @@ACTIVATE_CUSTOMER_RESOURCE = "/customers/" + @@CUSTOMER_ID + "/activate"
      @@CREATE_ADDRESS_RESOURCE =  "/customers/" + @@CUSTOMER_ID + "/addresses"
      @@GET_ADDRESSES_RESOURCE = "/customers/" + @@CUSTOMER_ID + "/addresses"
      @@GET_ADDRESS_RESOURCE =  "/customers/" + @@CUSTOMER_ID + "/addresses/" + @@ADDRESS_ID
      @@DELETE_ADDRESS_RESOURCE =  "/customers/" + @@CUSTOMER_ID + "/addresses/" + @@ADDRESS_ID
      @@UPDATE_ADDRESS_RESOURCE =  "/customers/" + @@CUSTOMER_ID + "/addresses/" + @@ADDRESS_ID
      @@GET_ADDRESS_BY_ID_RESOURCE = "/addresses/" + @@ADDRESS_ID

      def initialize(client)
        @client = client
      end

      def create_customer(payload, headers={:content_type => :json})
        @client.post(@@CREATE_CUSTOMER_RESOURCE, payload, headers)

      end

      def get_customer_by_id(customer_id, headers={:content_type => :json})
        resource_location = @@GET_CUSTOMER_RESOURCE.sub(@@CUSTOMER_ID, customer_id.to_s)
        @client.get(resource_location, headers)
      end

      def find_customers(params, headers={:content_type => :json})
        resource_location = @@GET_CUSTOMERS_RESOURCE.sub(@@PARAM_STRING, params)
        @client.get(resource_location, headers)
      end

      def delete_customer_by_id(customer_id, headers={:content_type => :json})
        resource_location = @@DELETE_CUSTOMER_RESOURCE.sub(@@CUSTOMER_ID, customer_id.to_s)
        @client.delete_json(resource_location, headers)
      end

      def update_customer_by_id(customer_id, payload, headers={:content_type => :json})
        resource_location = @@UPDATE_CUSTOMER_RESOURCE.sub(@@CUSTOMER_ID, customer_id.to_s)
        @client.put(resource_location, payload, headers)
      end

      def activate_customer_by_id(customer_id, headers={:content_type => :json})
        resource_location = @@ACTIVATE_CUSTOMER_RESOURCE.sub(@@CUSTOMER_ID, customer_id.to_s)
        @client.put(resource_location, headers)
      end

      def create_addressfor_customer_id(customer_id, payload, headers={:content_type => :json})
        resource_location = @@CREATE_ADDRESS_RESOURCE.sub(@@CUSTOMER_ID, customer_id.to_s)
        @client.post(resource_location, payload, headers)
      end

      def get_addressesfor_customer_id(customer_id, headers={:content_type => :json})
        resource_location = @@GET_ADDRESSES_RESOURCE.sub(@@CUSTOMER_ID, customer_id.to_s)
        @client.get(resource_location, headers)
      end

      def get_address_by_id_for_customer_id(customer_id, address_id, headers={:content_type => :json})
        resource_location = @@GET_ADDRESS_RESOURCE.sub(@@CUSTOMER_ID, customer_id.to_s).sub(@@ADDRESS_ID, address_id.to_s)
        @client.get(resource_location, headers)
      end

      def update_address_by_id_for_customer_id(customer_id, address_id, payload, headers={:content_type => :json})
        resource_location = @@UPDATE_ADDRESS_RESOURCE.sub(@@CUSTOMER_ID, customer_id.to_s).sub(@@ADDRESS_ID, address_id.to_s)
        @client.put(resource_location, payload, headers)
      end

      def delete_address_by_id_for_customer_id(customer_id, address_id, headers={:content_type => :json})
        resource_location = @@DELETE_ADDRESS_RESOURCE.sub(@@CUSTOMER_ID, customer_id.to_s).sub(@@ADDRESS_ID, address_id.to_s)
        @client.delete_json(resource_location, headers)
      end

      def get_address_by_id(address_id, headers={:content_type => :json})
        resource_location = @@GET_ADDRESS_BY_ID_RESOURCE.sub(@@ADDRESS_ID, address_id.to_s)
        @client.get(resource_location, headers)
      end

    end
  end
end