require File.expand_path(File.dirname(__FILE__) + '/../utils/logger.rb')
module SC
  module Client
    class WarehouseClient
      include SC::TestCore::Logger
      @@WAREHOUSE_ID_TEMP  = "-WAREHOUSE_ID-"
      @@PENDING_RESERVATION_ID_TEMP = "-PENDING_RESERVATION_ID-"
      @@PICKLIST_ID_TEMP = "-PICKLIST_ID-"
      @@PICKLIST_ITEM_ID_TEMP = "-PICKLIST_ITEM_ID-"
      @@LOCATION_TYPE = "-LOCATION_TYPE-"
      @@GRN_ID_TEMP = "-GRN_ID-"
      @@PUTLIST_ID_TEMP = "-PUTLIST_ID-"
      @@ORDER_ITEM_ID_TEMP = "-ORDER_ITEM_ID-"
      @@ADDRESS_IDS = "-ADDRESS_IDS-"

      @@CREATE_PENDING_RESERVATIONS       = "/warehouses/"+@@WAREHOUSE_ID_TEMP+"/pending_reservations"
      @@CREATE_PENDING_RESERVATIONS_BULK  = "/pending_reservations"
      @@GET_PENDING_RESERVATION_RESOURCE  = "/warehouses/"+@@WAREHOUSE_ID_TEMP+"/pending_reservations/"+@@PENDING_RESERVATION_ID_TEMP

      @@GET_PICKABLES_RESOURCE   = "/warehouses/"+@@WAREHOUSE_ID_TEMP+"/ship_groups/count"
      @@CREATE_PICKLISTS_RESOURCE   = "/warehouses/"+@@WAREHOUSE_ID_TEMP+"/picklists"
      @@CREATE_TRANSFER_LISTS_RESOURCE   = "/warehouses/"+@@WAREHOUSE_ID_TEMP+"/transfer_lists"
      @@PICK_PICKLISTS_RESOURCE   = "/warehouses/"+@@WAREHOUSE_ID_TEMP+"/picklists/"+@@PICKLIST_ID_TEMP+"/pick"
      @@GET_PICKLISTS_RESOURCE   = "/warehouses/"+@@WAREHOUSE_ID_TEMP+"/picklists/"+@@PICKLIST_ID_TEMP
      @@PICK_PICKLIST_ITEMS_RESOURCE   = "/warehouses/"+@@WAREHOUSE_ID_TEMP+"/picklists/"+@@PICKLIST_ID_TEMP+"/picklist_items/"+@@PICKLIST_ITEM_ID_TEMP+"/pick"
      @@GET_PICKLIST_ITEMS_RESOURCE   = "/warehouses/"+@@WAREHOUSE_ID_TEMP+"/picklists/"+@@PICKLIST_ID_TEMP+"/picklist_items/"+@@PICKLIST_ITEM_ID_TEMP

      @@ADD_TO_INVENTORY_RESOURCE   = "/warehouses/"+@@WAREHOUSE_ID_TEMP+"/inventory"
      @@GET_INVENTORY_RESOURCE   = "/warehouses/"+@@WAREHOUSE_ID_TEMP+"/inventory"
      @@ASSIGN_SHELF_RESOURCE       = "/warehouses/"+@@WAREHOUSE_ID_TEMP+"/inventory/transfer_location"

      @@CREATE_PUTLIST_RESOURCE   = "/warehouses/"+@@WAREHOUSE_ID_TEMP+"/putlists"
      @@GET_PUTLIST_RESOURCE   = "/warehouses/"+@@WAREHOUSE_ID_TEMP+"/putlists/"+@@PUTLIST_ID_TEMP

      @@GET_GRN_RESOURCE   = "/warehouses/"+@@WAREHOUSE_ID_TEMP+"/goods_receipt_notes/"+@@GRN_ID_TEMP
      @@GET_ORDER_ITEM_STATUS_RESOURCE   = "/order_items/"+@@ORDER_ITEM_ID_TEMP+"/status?address_ids="+@@ADDRESS_IDS
      @@CANCEL_ORDER_ITEM_RESOURCE   = "/order_items/"+@@ORDER_ITEM_ID_TEMP+"/cancel"

      @@SHIPMENT_ID_TEMP  = "-SHIPMENT_ID-"
      @@CONFIRM_RESOURCE  = "/shipments/"+@@SHIPMENT_ID_TEMP+"/confirm"

      def initialize(client = nil)
        @client = client
      end

      def get_warehouse_address(warehouse_id)
        resource  = "/warehouses/#{warehouse_id}/address"
        logger.info  "\nGetting warehouse addresses using resource #{resource}"
        response  = @client.get(resource, headers={})
        logger.info  "Get warehouse addresses Response #{response.inspect}"
        return response
      end

      #Warehouse Operations
      def create_pending_reservations(warehouse_id,pending_reservations_json_payload, headers={:content_type => :json})
        respurce  = @@CREATE_PENDING_RESERVATIONS.sub(@@WAREHOUSE_ID_TEMP,warehouse_id)
        logger.info  "\nCreating Pending Reservations with resource #{respurce} payload #{pending_reservations_json_payload}"
        response  = @client.post(respurce ,
                                 pending_reservations_json_payload.to_json, headers)
        logger.info  "Create Pending Reservation Response #{response.inspect}"
        return response
      end

      def create_bulk_pending_reservations(bulk_pending_reservations_json_payload, headers={:content_type => :json})
        respurce  = @@CREATE_PENDING_RESERVATIONS_BULK
        logger.info  "\nCreating bulk Pending Reservations with resource #{respurce} payload #{bulk_pending_reservations_json_payload}"
        response  = @client.post(respurce ,
                                 bulk_pending_reservations_json_payload.to_json, headers)
        logger.info  "Create Pending Reservation Response #{response.inspect}"
        return response
      end

      def create_bulk_reservations(payload, headers={:content_type => :json})
        respurce  = "/inventory/reserve"
        logger.info  "\nCreating bulk Reservations with resource #{respurce} payload #{payload}"
        response  = @client.post(respurce ,
                                 payload.to_json, headers)
        logger.info  "Create Reservation Response #{response.inspect}"
        return response
      end

      def get_pending_reservation(pending_reservation_id,warehouse_id)
        resource  = @@GET_PENDING_RESERVATION_RESOURCE.
            sub(@@WAREHOUSE_ID_TEMP,warehouse_id).
            sub(@@PENDING_RESERVATION_ID_TEMP,pending_reservation_id.to_s)
        logger.info  "\nGetting pending_reservation using resource #{resource}"
        response  = @client.get(resource, headers={})
        logger.info  "Get pending_reservation Response #{response.inspect}"
        return response
      end

      def add_to_inventory(warehouse_id,add_to_inventory_json_payload, headers={:content_type => :json})
        logger.info  "\nAdd to inventory with payload #{add_to_inventory_json_payload}"
        resource  = @@ADD_TO_INVENTORY_RESOURCE.sub(@@WAREHOUSE_ID_TEMP,warehouse_id)
        logger.info  "Resource #{resource}"
        response  = @client.post(resource ,
                                 add_to_inventory_json_payload.to_json, headers)
        logger.info  "Add To Inventory Response #{response.inspect}"
        return response
      end

      def get_inventory(warehouse_id,fsn,sku)
        resource  = @@GET_INVENTORY_RESOURCE.sub(@@WAREHOUSE_ID_TEMP,warehouse_id)+"?fsn="+fsn+"&sku="+sku
        logger.info  "\nGetting inventory using #{resource}"

        response  = @client.get(resource, headers={})
        logger.info  "Get inventory Response #{response.inspect}"
        return response
      end

      def get_GRN(warehouse_id,grn_id)
        resource  = @@GET_GRN_RESOURCE.
            sub(@@WAREHOUSE_ID_TEMP,warehouse_id).
            sub(@@GRN_ID_TEMP,grn_id.to_s)
        logger.info  "\nGetting GRN using #{resource}"

        response  = @client.get(resource, headers={})
        logger.info  "Get GRN Response #{response.inspect}"
        return response
      end

      def assign_shelf(warehouse_id,assign_shelf_json_payload, headers={:content_type => :json})
        resource  = @@ASSIGN_SHELF_RESOURCE.sub(@@WAREHOUSE_ID_TEMP,warehouse_id)
        logger.info  "Resource #{resource}"
        logger.info  "\nAssign Shelf with payload #{assign_shelf_json_payload}"
        response  = @client.post(resource ,
                                 assign_shelf_json_payload.to_json, headers)
        logger.info  "Assign Shelf Response #{response.inspect}"
        return response
      end

      def create_putlist(warehouse_id, payload, headers={:content_type => :json})
        resource  = @@CREATE_PUTLIST_RESOURCE.sub(@@WAREHOUSE_ID_TEMP,warehouse_id)
        logger.info  "Resource #{resource}"
        logger.info  "\nCreating Putlist with payload #{payload}"
        response  = @client.post(resource ,
                                 payload.to_json,
                                 headers)
        logger.info  "Create Putlist Response #{response.inspect}"
        return response
      end

      def update_putlist(warehouse_id, putlist_id, payload, headers={:content_type => :json})
        resource  = @@GET_PUTLIST_RESOURCE.
            sub(@@WAREHOUSE_ID_TEMP,warehouse_id).
            sub(@@PUTLIST_ID_TEMP,putlist_id.to_s)
        logger.info  "Resource #{resource}"
        logger.info  "\nUpdate Putlist with payload #{payload}"
        response  = @client.post(resource ,
                                 payload.to_json,
                                 headers)
        logger.info  "Update Putlist Response #{response.inspect}"
        return response
      end

      def update_putlist_item(warehouse_id, putlist_id, putlist_item_id, payload, headers={:content_type => :json})
        resource  = "/warehouses/#{warehouse_id}/putlists/#{putlist_id.to_s}/putlist_items/#{putlist_item_id.to_s}"
        logger.info  "Resource #{resource}"
        logger.info  "\nUpdate Putlist Item with payload #{payload}"
        response  = @client.post(resource ,
                                 payload.to_json,
                                 headers)
        logger.info  "Update Putlist Item Response #{response.inspect}"
        return response
      end

      def get_putlist(warehouse_id,putlist_id, suggestions = nil)
        resource  = @@GET_PUTLIST_RESOURCE.
            sub(@@WAREHOUSE_ID_TEMP,warehouse_id).
            sub(@@PUTLIST_ID_TEMP,putlist_id.to_s)
        resource += "?suggestions=#{suggestions}" if not suggestions.nil?

        logger.info  "\nGetting Putlist using resource #{resource}"

        response  = @client.get(resource, headers={})
        logger.info  "Get Putlist Response #{response.inspect}"
        return response
      end

      def get_putlist_item(warehouse_id,putlist_id,putlist_item_id)
        resource  = "/warehouses/#{warehouse_id}/putlists/#{putlist_id.to_s}/putlist_items/#{putlist_item_id.to_s}"
        logger.info  "\nGetting Putlist Item using resource #{resource}"

        response  = @client.get(resource, headers={})
        logger.info  "Get Putlist Item Response #{response.inspect}"
        return response
      end

      def get_ATP(payload, headers={:content_type => :json})
        resource  = "/inventory"
        logger.info  "Resource #{resource}"
        logger.info  "\nGetting ATP with payload #{payload}"
        response  = @client.post(resource,payload.to_json)
        logger.info  "Get ATP Response #{response.inspect}"
        return response
      end

      def get_pickables(warehouse_id, location_type, processing_method = "#{location_type}_picklist")
        resource  = @@GET_PICKABLES_RESOURCE.
            sub(@@WAREHOUSE_ID_TEMP,warehouse_id)
        if location_type
          resource  += "?location_type=#{location_type}&processing_method=#{processing_method}"
        end

        logger.info  "\nGetting Pickables using resource #{resource}"

        response  = @client.get(resource, headers={})
        logger.info  "Get Picklist Response #{response.inspect}"
        return response
      end

      def create_picklist(warehouse_id,create_picklist_json_payload, headers={:content_type => :json})
        logger.info  "\nCreating Picklist with payload #{create_picklist_json_payload}"
        resource  = @@CREATE_PICKLISTS_RESOURCE.sub(@@WAREHOUSE_ID_TEMP,warehouse_id)
        logger.info  "Resource #{resource}"
        response  = @client.post(resource,
                                 create_picklist_json_payload.to_json, headers)
        logger.info  "Create Picklist Response #{response.inspect}"
        return response
      end

      def create_transfer_list(warehouse_id, create_transfer_list_json_payload, headers={:content_type => :json})

        resource  = @@CREATE_TRANSFER_LISTS_RESOURCE.sub(@@WAREHOUSE_ID_TEMP,warehouse_id)

        response  = @client.post(resource,
                                 create_transfer_list_json_payload.to_json, headers)

        return response
      end

      def pick_picklist(warehouse_id,picklist_id,  headers={:content_type => :json})
        resource  = @@PICK_PICKLISTS_RESOURCE.sub(@@WAREHOUSE_ID_TEMP,warehouse_id).
            sub(@@PICKLIST_ID_TEMP,picklist_id.to_s)
        logger.info  "Pick picklist using resource #{resource}"
        response  = @client.post(resource ,
                                 {}, headers)
        logger.info  "Pick Picklist Response #{response.inspect}"
        return response
      end

      def pick_picklist_item(warehouse_id, picklist_id, picklist_item_id,  quantity, headers={:content_type => :json})
        resource  = @@PICK_PICKLIST_ITEMS_RESOURCE.sub(@@WAREHOUSE_ID_TEMP,warehouse_id).
            sub(@@PICKLIST_ID_TEMP,picklist_id.to_s).
            sub(@@PICKLIST_ITEM_ID_TEMP,picklist_item_id.to_s)+"/#{quantity}"

        logger.info  "Pick picklist item using resource #{resource}"
        response  = @client.post(resource ,
                                 {}, headers)
        logger.info  "Pick Picklist item Response #{response.inspect}"
        return response
      end

      def get_picklist(warehouse_id,picklist_id)
        resource  = @@GET_PICKLISTS_RESOURCE.
            sub(@@WAREHOUSE_ID_TEMP,warehouse_id).
            sub(@@PICKLIST_ID_TEMP,picklist_id.to_s)
        logger.info  "\nGetting Picklist using resource #{resource}"

        response  = @client.get(resource, headers={})
        logger.info  "Get Picklist Response #{response.inspect}"
        return response
      end

      def get_picklist_item(warehouse_id,picklist_id,picklist_item_id, quantity)
        resource  = @@GET_PICKLIST_ITEMS_RESOURCE .
            sub(@@WAREHOUSE_ID_TEMP,warehouse_id).
            sub(@@PICKLIST_ID_TEMP,picklist_id.to_s).
            sub(@@PICKLIST_ITEM_ID_TEMP,picklist_item_id.to_s)
        logger.info  "\nGetting Picklist item using resource #{resource}"

        response  = @client.get(resource, headers={})
        logger.info  "Get Picklist item Response #{response.inspect}"
        return response
      end

      def get_order_item_status(order_item_id, address_ids_str)
        resource  = @@GET_ORDER_ITEM_STATUS_RESOURCE .
            sub(@@ORDER_ITEM_ID_TEMP,order_item_id.to_s).
            sub(@@ADDRESS_IDS,address_ids_str)
        logger.info  "\nGetting Order item status using resource #{resource}"

        response  = @client.get(resource, headers={})
        logger.info  "Get Order item status Response #{response.inspect}"
        return response
      end

      def cancel_order_item(order_item_id, cancellation_payload, headers={:content_type => :json})
        resource  = @@CANCEL_ORDER_ITEM_RESOURCE.
            sub(@@ORDER_ITEM_ID_TEMP,order_item_id.to_s)
        logger.info  "\nCancel Order item using resource #{resource} and payload #{cancellation_payload.to_json}"

        response  = @client.post(resource ,cancellation_payload.to_json, headers)
        logger.info  "Cancel Order item Response #{response.inspect}"
        return response
      end

      def confirm_shipment(shipment_id, json_confirm_shipment_payload, headers={:content_type => :json})
        logger.info  "\nConfirming Shipment #{shipment_id} with payload #{json_confirm_shipment_payload} "
        response =  @client.post(@@CONFIRM_RESOURCE.sub(@@SHIPMENT_ID_TEMP,shipment_id.to_s),json_confirm_shipment_payload.to_json, headers)
        logger.info  "Confirm Shipment Response #{response.inspect}"
        return response
      end

      def reprocess_shipment(old_shipment_id, headers={:content_type => :json})
        logger.info  "\nReprocess Shipment #{old_shipment_id} "
        resource = "/shipments/#{old_shipment_id}/reprocess"
        response =  @client.post(resource,{}, headers)
        logger.info  "Reprocess Shipment Response #{response.inspect}"
        return response
      end

      def get_pending_reservations_by_order_id(order_id)
        resource  = "/pending_reservations?order_item_id=#{order_id}"
        logger.info  "\nGetting pending reservations using resource #{resource}"
        response  = @client.get(resource, headers={})
        logger.info  "Get pending reservation status Response #{response.inspect}"
        return response
      end

    end
  end
end
