require_relative '../constants/http_status_codes'
require_relative '../factory/shipping_factory'
#Mixin of WarehouseOperations requires
#   : warehouse_client function implementation in the class which is mixin WarehouseOperations
#   : shelves function to return hash of shelf hashies
#   : store_locations function to return hash of shelf hashies

module SC
  module Operations
    module ShippingOperations
      include SC::Constants::HttpStatusCodes
      include SC::Factory
      def create_shipment(shipment_body)
        response = shipping_client.create_shipment(shipment_body)
        if response.code == 201
          return response, JSON(response.body)["shipment"]["id"], JSON(response.body)["shipment"]["shipment_items"].collect {|shipment_item| shipment_item["id"]}
        end
        response
      end

      def create_shipment!(shipment_body)
        response, shipment_id, shipment_item_ids = create_shipment(shipment_body)
        raise("Create Shipment Failed with Reason #{response.body}") if response.code != STATUS_CREATED
        return response, shipment_id, shipment_item_ids
      end

      def get_shipment(shipment_id)
        shipping_client.get_shipment(shipment_id)
      end

      def confirm_shipment(shipment_id, weight, tracking_id)
        shipping_client.confirm_shipment(shipment_id, ShippingFactory.create_confirm_shipment_body(weight,tracking_id))
      end

      def confirm_shipment!(shipment_id, weight, tracking_id)
        response = confirm_shipment(shipment_id, weight, tracking_id)
        raise("Confirm Shipment Failed with Reason #{response.body}") if response.code != STATUS_UPDATED
        response
      end

      def get_shipment_items_status(order_item_id, address_ids_str)
        shipping_client.get_shipment_items_status(order_item_id,address_ids_str)
      end

      def cancel_shipment(shipment_id)
        shipping_client.cancel_shipment(shipment_id)
      end

      def cancel_shipment!(shipment_id)
        response = cancel_shipment(shipment_id)
        raise("Cancel Shipment failed with reason #{response.body}") if response.code != STATUS_DELETED
        response
      end
    end
  end
end
