require 'socket'
module SC
  module Utils
    module HelperFunctions
      def self.wait_for_socket!(host, port,max_wait,granular_wait=0.5)
        total_wait = 0
        while(total_wait < max_wait)
          return if is_socket_up?(host, port)[0]
          puts "Socket #{host}:#{port} not up. Sleeping for #{granular_wait} seconds. Total Wait #{total_wait}"
          sleep granular_wait
          total_wait += granular_wait
        end
        raise "Socket #{host}:#{port} not up in #{max_wait} seconds" if not is_socket_up?(host, port)[0]
      end

      def has_json?(json)
        json.each do |key, value|
          return false if self[key] != value
        end
        true
      end

      private
      def self.is_socket_up?(host, port)
        s = TCPSocket.open(host, port)
        s.close               # Close the socket when done
        puts "Socket #{host}:#{port} is up."
        return true,""
      rescue => e
        return false, e.message
      end

      def clear_matching_files(dir,file_name_pattern)
        @log.info("Cleaning files with pattern #{file_name_pattern} in dir #{dir}")
        Dir.new(dir).entries.each do |entry|
          File.delete(dir+"/"+entry) if not (entry =~ file_name_pattern).nil?
        end
      end

      def wait_for_file(dir,file_name,max_wait=60, granular_wait=1.0)
        file_path = dir+"/"+file_name
        @log.info("Waiting for the presence of file #{file_path}")
        if wait_until(max_wait, granular_wait) {File.exist?(dir+"/"+file_name)}
          @log.info("Waiting for file #{file_path} size to stabilize")
          initial_file_size = File.size(file_path)
          return dir+"/"+file_name if wait_until(max_wait, granular_wait, 1.0) do
            File.size(file_path) == initial_file_size
            initial_file_size = File.size(file_path)
           end
        end
        return nil
      end

      def wait_until(max_wait=60, granular_delay=0.1,initial_delay = 0.0, &block)
        sleep initial_delay
        total_wait = 0
        while(not block.call)
          break if total_wait >= max_wait
          sleep granular_delay
          total_wait += granular_delay
          puts "Total Delay #{total_wait}"
        end
        block.call
      end

    end
  end
end
